%The code produces a single realization of the following chemical model.
%The key point is that 2 streams of unit-exponentials are provided and 
%we use those to make the Poisson processes.  We do this so we can
%implement the common reaction path method.
%
%
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. We take
%          the derivative with respect to kb.
%
% Usage [x,nrv]= NRM_CRP(tend,xin,parameters,RVs)
%
%               parameters = the parameters of the model.
%                            It is a vector, but only uses the 5th and 6th
%                            elements (the others are for the compartment)
%               tend = end time. Real.
%               xin = initial condition.  1 x 1vector.  S
%               RVs = inputed unit exponential random variables.  This will
%                     be an array of size 2 x number_exponentials
%
%
%The outpus are the following:
%
%                     x = output vector (final state)
%
%                     nrv = number of random variables utilized.
%
%Written by David F. Anderson, June 25th, 2024.

function [x,nrv]= NRM_CRP(tend,xin,parameters,RVs)

%track the random variables used
RVCount = 1;

%max number of steps
N=25;

kb=parameters(5);
kd=parameters(6);

%Initialize intensities.
lambda = zeros([2,1]);

x = xin;
times=zeros([1,N+1]);
nrv=0;

%Internal times of the integrated intensity functions.
T= zeros([2,1]);

%Times for the next unit Poisson process in the stream.
%Pointer(k) will tell us where each stream should take NEXT.
Pointer = ones([2,1]);
P= zeros([2,1]);
for k = 1:2
    P(k) = RVs(k,Pointer(k));
    Pointer(k) = Pointer(k)+1;
    nrv=nrv+1;
end

%Initialize Deltas (internal time until next reaction)
IntensityDelta = zeros([2,1]);
for i = 1:N
    
    lambda(1) = kb;
    lambda(2) = kd*x(1);
    
    %I could do this as vectors.  Could change later if I want.    
    for k = 1:2
        if lambda(k)>0
            IntensityDelta(k) = (P(k) - T(k))/lambda(k);
        else
            IntensityDelta(k) = Inf;
        end
    end

    %Find the minimum and its index.
    if IntensityDelta(1)<IntensityDelta(2)
        minDelta = IntensityDelta(1);
        indexMin = 1;
    else
        minDelta = IntensityDelta(2);
        indexMin = 2;
    end
    %[minDelta, indexMin] = min(IntensityDelta);

    %update time.
    times(1,i+1)=times(1,i)+minDelta;
    
    %If the next reaction happens after tend, we are done.
    if times(1,i+1)>tend
        break
    
        %Otherwise, find which reaction happened and update.
    else

        %Update all the integrated intensities.
        for k = 1:2
            T(k) = T(k) + lambda(k)*minDelta;
        end
        
        %Only one reaction needs a new unit-exponential.  Do that and add
        %to it's pointer.  
        P(indexMin) = P(indexMin)+RVs(indexMin,Pointer(indexMin));
        
        Pointer(indexMin) = Pointer(indexMin)+1;
        
        nrv = nrv+1;
        %Update the reaction.  (Would be nice to have reaction vectors)
        
        if indexMin == 1
            x(1) = x(1) + 1;

        else
            x(1) = x(1) - 1;
        end
        
    end
    
end
if i >= N
    'Too big in CRP Path generation'
end