%
%  main.m	This is the outer code to call for multiple realizations of the
%           compartment model using coupled compartments, and independent paths.  
%           The model is the following
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. We take
%          the derivative with respect to kb.
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
%  usage:	mainInd(n,tend,h)
%
%               n = number of times to run to get stats.
%
%               tend = time of simulation
%
%               h = size of difference
%
%  example:	mainInd(1000,500,1/10)
%
%  The output will be the mean and estimator variance of the statistic of
%  choice. I plan for that statistic to be the total (across all cells)
%  number of S.
%
%
%Written by David F. Anderson, June 4th, 2024.

function [] = mainInd(n,tend,h)

%For timing purposes
tic

%Set the maximum number of compartments (for pre-allocation).  This will be
%case specific.  For the model I'm using we should never even come close to
%this number of compartments.
CompartmentMax = 200;

%Set the number of timepoints we want (for plotting variances).  Will plot
%a total of number_timepoints+1.
number_timepoints = 200;
%This is the time frame increment
del = tend/number_timepoints;
%Placeholders for the different MC (number of compartments)
OutputMC1 = zeros([number_timepoints+1,1]);
OutputMC2 = zeros([number_timepoints+1,1]);

%The state of the system is now a 1 dimensional array.  It will be  size
%1 x CompartmentMax x number_timepoints+1,
%with the 1 giving the count of species S, the CompartmentMax giving the
%compartment number (but we only care about it up to MC) and
%number_timepoints+1 giving time. 
Xnominal = zeros([1, CompartmentMax, number_timepoints+1]);
Xperturbed = zeros([1, CompartmentMax, number_timepoints+1]);

%Set the parameters
%The first four are the compartment parameters.
KI = 1;
KE = 0.01;
KF = 0;
KC = 0;
%The next 2 are the chemical parameters.
kb = 1;
kd = 0.1;
%The next is lambda (mean of Poisson for input distribution)
lambda = 10;

%If I want to fix the seed of the random number generator
%rng(1234)

%I need the total number of S for each of the n iterations (first
%dimension) and for each of the different time points (second dimension).
TotalSNominal = zeros([n,number_timepoints+1]);
TotalSPerturbed = zeros([n,number_timepoints+1]);

%For the number of compartments
TotalCompartments = zeros([n,number_timepoints+1]);

%The difference (divided by h) of the two realizations.
deriv = zeros([n,number_timepoints+1]);

%EstDeriv will be the actual estimated derivative at each timepoint.
EstDeriv = zeros([number_timepoints+1,1]);
%EstVar will give the estimated variance at each timepoint.
EstVar = zeros([number_timepoints+1,1]);
%I would like to plot the estimated number of total S
EstS = zeros([number_timepoints+1,1]);
%For plotting the mean number of compartments
EstCompartments = zeros([number_timepoints+1,1]);


%Count the number of RVs used. Not sure I even want this.
RV_Count = 0;

% parameters for each model (nominal and perturbed)
parameters = [KI, KE, KF, KC, kb, kd, lambda];
parametersPerturbed =  [KI, KE, KF, KC, kb+h, kd, lambda];

% main loop
for j = 1:n
    %The two key objects that must be returned by a single realization are
    %following:
    %
    % 1. The value of MC (number of compartments) at each of the relevant time points.
    %
    % 2. The state of the RNIC model at each of the relevant time points.
    % Specifically, this means giving the counts of each species in the
    % MC(t) compartments, where MC(t) is the number of compartments at time
    % t, for all relevant t.
    %
    %
    %Also, but slightly less important, is NRVs, the number of random variables used.
    %
    %This will call to get both realizations: perturbed and nominal.
    %I am going to initialize everything with MC = 1 and all states set to
    %0.  That's also what I will do if a new cell appears (easy to change)
    
    [OutputMC1, Xnominal,OutputMC2, Xperturbed, NRVs1] = SingleRealizationFullPath_Shell_Ind(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints);

    %We should have that OutpuMC1=OutputMC2.  If not something went wrong
    if OutputMC1 ~= OutputMC2
        'Something is wrong.  Should have OutputMC1=OutputMC2'
        break
    end
    %Just to see how far along we are with the script.
    if mod(j,n/10)==0
        j
        toc
    end

    %Now to process data.  Will get the total number of S at each
    %time.
    for t = 1:number_timepoints+1
        %sum over all the relevant compartments.
        TotalSNominal(j, t) = sum(Xnominal(1,1:OutputMC1(t),t));
    end
    for t = 1:number_timepoints+1
        %sum over all the relevant compartments.
        TotalSPerturbed(j, t) = sum(Xperturbed(1,1:OutputMC2(t),t));
    end
    for t = 1:number_timepoints+1
       deriv(j,t) =  (1/h)*(TotalSPerturbed(j, t)-TotalSNominal(j, t));
    end
    
    for t = 1:number_timepoints+1
        TotalCompartments(j,t) = OutputMC1(t);
    end


    RV_Count = RV_Count + NRVs1;
end

%Output the number of random variables used.
RV_Count

for t = 1:number_timepoints+1
    EstDeriv(t) = mean(deriv(:,t));
    EstVar(t) = var(deriv(:,t))/n;
    EstS(t) = mean(TotalSNominal(:,t));
    EstCompartments(t) = mean(TotalCompartments(:,t));
end

toc

Tp = zeros([number_timepoints+1,1]);
for m = 2:number_timepoints+1
    Tp(m) = Tp(m-1) + del;
end

%Just going to plot from the first compartment

%This is the state of the system from a single compartment.  Not sure
%what it gives us actually.  Just going to plot from the first compartment
Xnom1 = zeros([number_timepoints+1,1]);
for k = 1:number_timepoints+1
    Xnom1(k) = Xnominal(1,1,k);
end
figure
plot(Tp,Xnom1,'blue')
legend('Sample S')


%Now the variance plot
figure
plot(Tp,EstVar)
legend('Independent Estimator Variance')

figure
plot(Tp, EstDeriv)
legend('Estimated Derivative of total S')

figure 
plot(Tp, EstS)
legend('Estimated mean of total S')

figure 
plot(Tp, EstCompartments)
legend('Estimated mean of total compartments')

figure 
plot(Tp, OutputMC1)
legend('Realization of number of compartments from one run')


%sumtime

% CFD_data = vdata;
% save_CFD = ['CFD' '.mat'];
% save(save_CFD,'CFD_data');

end %The program

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
