%The code produces a single realization of coupled paths for the following RNIC model.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. We take
%          the derivative with respect to kb.
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
% Usage [MCoutput,Xoutput,MCoutputPert,XoutputPert,NRV]= SingleRealizationFullPath_Shell_Split(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints)
%
%
%               parametersPerturbed = the perturbed parameters of the model (listed above). Vector.
%               parameters = the nominal parameters of the model (listed above). Vector.
%               tend = end time. Real.
%               CompartmentMax = maximum number of compartments.  Integer.
%               number_timepoints = total number of timepoints for the purposes of plotting.  Integer.
%
%
%The outpus are the following:
%
%                     MCoutput = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time points.
%
%                     Xoutput = state of system at relevant times for nominal model.  Vector of size
%                     1 x CompartmentMax x (number_timepoints + 1)
%
%                     MCoutputPert = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time
%                     points for the perturbed model.
%
%                     XoutputPert = state of system at relevant times for perturbed model.  Vector of size
%                     1 x CompartmentMax x (number_timepoints + 1)
%
%                     NRV = number of random variables utilized.
%
%Written by David F. Anderson, June 25th, 2024.


function [MCoutput,Xoutput,MCoutputPert,XoutputPert,NRV]= SingleRealizationFullPath_Shell_Split(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints)

%Need to have the times that I will be updating my outputs. This is the
%amount of time I must simulate.
del = tend/number_timepoints;
OutputTimes = zeros([number_timepoints+1,1]);
for ell = 1:number_timepoints
    OutputTimes(ell+1) = OutputTimes(ell) + del;
end

%State of the system at relevant time-points.
Xoutput = zeros([1,CompartmentMax,number_timepoints+1]);
XoutputPert = zeros([1,CompartmentMax,number_timepoints+1]);

MCoutput = zeros([number_timepoints+1,1]);
MCoutputPert = zeros([number_timepoints+1,1]);

%INITIALIZE
MCoutput(1)=0;
MCoutputPert(1) = 0;

Xinitial=zeros([1,1]);
XPertInitial=zeros([1,1]);

for k = 1:MCoutput(1)
    Xoutput(:,k,1) = Xinitial;
end
for k = 1:MCoutputPert(1)
    XoutputPert(:,k,1) = XPertInitial;
end

%How many random variables used
NRV=0;

%Main Loop to get the coupled paths.
for ell = 1:number_timepoints
    %MCoutput_new should be a single number
    %MCoutputPert_new should be a single number
    %Xoutput_new should be a vector of size 1 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %XoutputPert_new should be a vector of size 1 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %NRV_new is the number of random variables used in this single step (of
    %size del)
    [MCoutput_new,Xoutput_new,MCoutputPert_new,XoutputPert_new,NRV_new]= SingleRealizationSingleTimeCompShell_Split(MCoutput(ell),Xoutput(:,:,ell),MCoutputPert(ell), XoutputPert(:,:,ell),parameters,parametersPerturbed,del);
    MCoutput(ell+1)=MCoutput_new;
    MCoutputPert(ell+1) = MCoutputPert_new;
    for k = 1:MCoutput_new
        Xoutput(:,k,ell+1) = Xoutput_new(:,k);
    end
    for k = 1:MCoutputPert_new
        XoutputPert(:,k,ell+1) = XoutputPert_new(:,k);
    end
    NRV = NRV + NRV_new;
end


