%The code produces a single realization of the following RNIC model.
%It uses the unit-exponential random variables that come from RVs.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. 
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_Split(MCInit,Xinit,parameters,tend,RVsExp,RVsDelete,RVsFrag,RVsCoag, PointerExp, PointerDel, PointerFrag, PointerCoag)
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               Xinit = vector of size 1 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%               RVs = stream of unit-exponentials.  Vector of size 6 x
%               (something)
%               PointerExp = vector of size 4 x 1 pointing to the next RV used
%               from RVs for each component reaction.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     1 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%                     PointerExpNew=the new pointer for the unit-exponentials.
%                     PointerDelNew=the new pointer for the deletions.
%                     PointerFragNew=the new pointer for the fragmentations.
%                     PointerCoagNew=the new pointer for the coagulations.
%
%We simulate the compartment events using the CRP coupling.  
%
%Written by David F. Anderson, June 26th, 2024.

function [MC,X,NRV,PointerExp,PointerDel,PointerFrag,PointerCoag,PointerBinom,PointerInput, T,P]= SingleRealizationSingleTimeCompShell_CRP(MCInit,Xinit,parameters,tend,RVsExp,RVsDelete,RVsFrag,RVsCoag, RVsBinom, RVsInput, PointerExpIn, PointerDelIn, PointerFragIn, PointerCoagIn, PointerBinomIn, PointerInputIn, Tin,Pin)
%This is for a perturbation in the compartment model.  Trying KF at first.


%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^3;

%Update stuff
T = Tin;
P = Pin;


PointerExp = PointerExpIn;
PointerDel = PointerDelIn;
PointerFrag = PointerFragIn;
PointerCoag = PointerCoagIn;
PointerBinom = PointerBinomIn;
PointerInput = PointerInputIn;

%State of the system.
X = Xinit;
MC = MCInit;

%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
lambdaInput = parameters(7);


%Rates for the  compartments.
lambda = zeros([4,1]);


%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;


%Initialize Deltas (internal time until next reaction)
IntensityDelta = zeros([4,1]);

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input (nominal process)
    lambda(1) = KI;
    %Rate of compartment exit (nominal process)
    lambda(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C.  (nominal process)
    lambda(3) = KF*MC;
    %Rate of Coagulation (nominal process)
    lambda(4) = KC*MC*(MC-1);

    %I could do this as vectors.  Could change later if I want.    
    for k = 1:4
        if lambda(k)>0
            IntensityDelta(k) = (P(k) - T(k))/lambda(k);
        else
            IntensityDelta(k) = Inf;
        end
    end
    
    %Find the minimum and its index.
    [minDelta, indexMin] = min(IntensityDelta);

    %update time.
    times(1,i+1)=times(1,i)+minDelta;
    
    
    %check if we went passed end time.

    if times(1,i+1) > tend
        times(1,i+1) = tend;
        CheckBreak=1;
    end
    
    
    %Now update the chemistry within the components.  No coupling here.

    %update extra unperturbed
    for k = 1:MC
        [x,nrvs] = StandardGillespie(times(i+1)-times(i),X(:,k),parameters);
        X(:,k) = x;
        NRV = NRV+nrvs;
    end


    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %Even though I'm breaking out, the Ts need to update.  The P does not because we don't actually see it yet.
        for k = 1:4
            T(k) = T(k) + lambda(k)*(times(1,i+1)-times(1,i));
        end
        break
        
    else
        
        
        %Update all the integrated intensities.
        for k = 1:4
            T(k) = T(k) + lambda(k)*minDelta;
        end
        
        %Only one reaction needs a new unit-exponential.  Do that and add
        %to it's pointer.
        P(indexMin) = P(indexMin)+RVsExp(indexMin,PointerExp(indexMin));
        
        PointerExp(indexMin) = PointerExp(indexMin)+1;
        
        NRV = NRV+1;
        
        %Find the indexMin
        if indexMin==1 %reaction 1, 0 --> C, appearance of compartment at end
            
            %New compartment with nothing in it.  
            InputValue = poissinv(RVsInput(PointerInput),lambdaInput);
            PointerInput=PointerInput+1;
            NRV = NRV+1;
            
            X(:,MC+1) = InputValue;
            
            %Update number of compartments
            MC = MC+1;
            
            

        elseif indexMin==2 %reaction 2, C --> 0, deletion
                
            
            ufind = RVsDelete(PointerDel);
            PointerDel=PointerDel+1;
            
            NRV = NRV+1;
            
            %Which compartments to be deleted
            IndexDel = floor(ufind*MC)+1;
            
            %update all before this index
            for ell = 1:IndexDel-1
                X(:,ell) = X(:,ell);
            end
            
            %Now shift everything down
            for ell = IndexDel:MC-1
                X(:,ell) = X(:,ell+1);
            end
            
            %Update number of compartments
            MC = MC-1;
            

        elseif indexMin==3 %reaction 3, C --> 2C, fragmentation.
            
            11111
            'Fragmentation?'
            %Pick which compartment fragments.  Note state dependent, so easy.
            ufind = RVsFrag(PointerFrag);
            PointerFrag=PointerFrag+1;
            NRV = NRV+1;
            
            %Which compartments to be deleted
            IndexFrag = floor(ufind*MC)+1;
            
            
            %Get the Ss remaining

            U1=RVsBinom(PointerBinom);
            PointerBinom = PointerBinom+1;
            NRV=NRV+1;
            
            SIndex =  myinv(U1,X(1,IndexFrag),1/2);
      
            
            %Add new compartment and update new compartment
            X(1,MC+1) = X(1,IndexFrag)-SIndex;
            
            %update IdexFrag
            X(1,IndexFrag) = SIndex;
            
            MC = MC+1;
            
            
        else %reaction 4, 2C --> C, coagulation
            
            222222
            'Coagulation?'
            
            %select two indices at random
            U1 = RVsCoag(PointerCoag);
            PointerCoag=PointerCoag+1;
            
            ri1 = floor(U1*MC)+1;
            NRV = NRV+1;
            
            %Now I need the second index.  From the set {1,...,MC}\{ri1}
            U2 = RVsCoag(PointerCoag);
            PointerCoag=PointerCoag+1;
            NRV=NRV+1;
            
            ri2 = floor(U2*(MC-1))+1;
            %that works if ri2<ri1.  If ri2 >= ri1, add one.
            if ri2 >= ri1
                ri2 = ri2+1;
            end
            
            %I will put the material in ri1;
            
            %collect in ri1 compartment.
            X(:,ri1)=X(:,ri1)+X(:,ri2);
            
            %Now I have to delete ri2 and shift everything down.
            for k = ri2:MC-1
                X(:,k) = X(:,k+1);
            end
            
            %update number of compartments.
            MC = MC-1;
        end
        
    end
end


if i >=NMax
    4444
    'over NMax'
end



end


function [b] = myinv(u,n,p)
%I'm assuming p= 1/2.
%b = binoinv(u,n);

if n <=12
    check=0;
    summ=0;
    for K = 0:n
        summ=summ+nchoosek(n,K)*(1/2)^n;
        if summ >= u
            check=1;
            b=K;
            break
        end
    end
    if check==0
        'what'
        n
        u
        b=n
    end
else
    mu=n/2;
    sigma=sqrt(n*p*(1-p));
    b=norminv(u,mu,sigma);
    b=round(b);
    if b < 0
        b=0;
    elseif b > n
        b=n;
    end
end
end

%toc


