%The code produces a single realization of the following RNIC model. The
%only change with the standard code is that a seed for the random number
%generator is inputed.
%
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. 
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
% Usage [MCoutput,Xoutput,NRV]= SingleRealizationFullPath_Shell_CRN(parameters,tend,CompartmentMax,number_timepoints,seed)
%
%               parameters = the parameters of the model (listed above). Vector.
%               tend = end time. Real.
%               CompartmentMax = maximum number of compartments.  Integer.
%               number_timepoints = total number of timepoints for the purposes of plotting.  Integer.
%               seed = integer.  Gives the seed of the random number
%               generator.
%
%
%The outpus are the following:
%
%                     MCoutput = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC (number of components) at different relevant time points.
%
%                     Xoutput = state of system at relevant times.  Vector of size
%                     1 x CompartmentMax x (number_timepoints + 1)
%
%                     NRV = number of random variables utilized.
%
%Written by David F. Anderson, June 25th, 2024.


function [MCoutput,Xoutput,NRV]= SingleRealizationFullPath_Shell_CRN(parameters,tend,CompartmentMax,number_timepoints,seed)

%Sets the seed of the random number generator. Should be the same as in
%independent paths after that.

rng(seed)

%Need to have the times that I will be updating my outputs. This is the
%amount of time I must simulate.
del = tend/number_timepoints;
OutputTimes = zeros([number_timepoints+1,1]);
for ell = 1:number_timepoints
    OutputTimes(ell+1) = OutputTimes(ell) + del;
end


%State of the system at relevant time-points.
Xoutput = zeros([1,CompartmentMax,number_timepoints+1]);
MCoutput = zeros([number_timepoints+1,1]);

%INITIALIZE
MCoutput(1)=0;
Xinitial=zeros([1,1]);
for k = 1:MCoutput(1)
    Xoutput(:,k,1) = Xinitial;
end

%How many random variables used
NRV=0;

%Main Loop to get the path.
for ell = 1:number_timepoints
    %MCoutput_new should be a single number
    %Xoutput_new should be a vector of size 1 x CompartmentMax (with only
    %the first MCoutput_new columms mattering
    %NRV_new is the number of random variables used in this single step (of
    %size del)
    [MCoutput_new,Xoutput_new,NRV_new]= SingleRealizationSingleTimeCompShell(MCoutput(ell),Xoutput(:,:,ell), parameters,del);
    MCoutput(ell+1)=MCoutput_new;
    for k = 1:MCoutput_new
        Xoutput(:,k,ell+1) = Xoutput_new(:,k);
    end
    NRV = NRV + NRV_new;
end
