%The code produces a single realization of the following RNIC model for the
%Girsanov method.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. We take
%          the derivative with respect to k_E.
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;

%
% Usage [MC,X,NRV,W]= SingleRealizationSingleTimeCompShell(MCInit,Xinit,parameters,tend)
%
%               MCInit = the starting number of compartments.  Integer.
%               Xinit = vector of size 1 x (CompartmentMax) giving initial
%                       condition (with MCInit)
%               parameters = the parameters of the model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count%  
%
%                     X = state of system at end time.  Vector of size
%                     1 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%                     W = weight function for Girsanov transformation.
%                     Importantly, this is coded for W to come from the
%                     chemical system (the derivative of a chemical
%                     parameter)
%
%We simulate the compartment events.  We then simulate using a call to
%Gillespie between.
%
%Written by David F. Anderson, June 26th, 2024.

function [MC,X,NRV,W]= SingleRealizationSingleTimeCompShell(MCInit,Xinit,parameters,tend)


%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^3;


%State of the system.
X = Xinit;
MC = MCInit;

%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
% kb = parameters(5);
% kd = parameters(6);
lambda = parameters(7);

%Rates for the compartments.
lambdaComp = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;

W = 0;

for i = 1:NMax
    
    %Rate of compartment input
    lambdaComp(1) = KI;
    %Rate of compartment exit
    lambdaComp(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C. 
    lambdaComp(3) = KF*MC;
    %Rate of Coagulation
    lambdaComp(4) = KC*MC*(MC-1);

    %Total rate for compartments
    TotalRate = lambdaComp(1)+lambdaComp(2)+lambdaComp(3)+lambdaComp(4);
    
    %Get time of next compartment event.  Standard method.
    r1 = rand;
    NRV=NRV+1;

    Delta = log(1/r1)/TotalRate;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.

    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %At this point, we have MC compartments.  So need to call a single
    %realization implementation for each of these comaprtments.  
    for k = 1:MC
        [x,nrvs] = StandardGillespie(times(i+1)-times(i),X(:,k),parameters);
        X(:,k) = x;
        NRV=NRV+nrvs;
    end

    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        W = W - (times(i+1)-times(i))*lambdaComp(2)/KE;
        break
    else

        W = W - (times(i+1)-times(i))*lambdaComp(2)/KE;
        r2=rand;
        NRV = NRV+1;

        if r2 < lambdaComp(1)/TotalRate %appearance of compartment at end

            %All previous compartments are the same
            X(:,1:MC) = X(:,1:MC);

            %New compartment Poisson(lambda).  
            
            X(:,MC+1) = poissrnd(lambda);
            NRV=NRV+1;
            %Update number of compartments
            MC = MC+1;

        elseif r2 < (lambdaComp(1)+lambdaComp(2))/TotalRate %deletion
            %Update W (since taking a derivative with respect to this rate)
            W = W + 1/KE;
            
            %find the one to be deleted.
            IndexDel = randi([1,MC]);
            NRV = NRV + 1;
            
            %update all before this index
            for ell = 1:IndexDel-1
                X(:,ell) = X(:,ell);
            end
            %Now shift everything down
            for ell = IndexDel:MC-1
                X(:,ell) = X(:,ell+1);
            end
            %Update number of compartments
            MC = MC-1;


        elseif r2<(lambdaComp(1)+lambdaComp(2)+lambdaComp(3))/TotalRate %fragmentation.

            'Fragmentation?'
            %Pick which compartment fragments.  Not state dependent, so easy.
            IndexFrag = randi([1,MC]);
            NRV=NRV+1;

            %Get the Ss remaining
            SIndex = binornd(X(1,IndexFrag),1/2);
            NRV = NRV+1;

            %update all compartments to be exactly the same
            for k = 1:MC
                X(:,k) = X(:,k);
            end

            %Add new compartment and update new compartment
            X(1,MC+1) = X(1,IndexFrag)-SIndex;

            %update IdexFrag
            X(1,IndexFrag) = SIndex;

            MC = MC+1;

        else %Do coagulation

            'Coagulation?'
            %select two indices at random
            ri1=randi([1 MC]);
            ri2=randi([1 MC]);
            NRV=NRV+2;

            while ri1==ri2
                ri2=randi([1 MC]);
                NRV=NRV+1;
            end
            %I will put the material in ri1;

            %First, update all to be the same
            for k = 1:MC
                X(:,k) = X(:,k);
            end

            %collect in ri1 compartment.
            X(:,ri1)=X(:,ri1)+X(:,ri2);

            %Now I have to delete ri2 and shift everything down.
            for k = ri2:MC-1
                X(:,k) = X(:,k+1);
            end

            %update number of compartments.
            MC = MC-1;
        end

    end
end

if i >=NMax
    'over NMax'
end
%toc



