%The code produces coupled realizations of the following chemical model.
%
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. 
%
%
% with both nominal and perturbed parameters.  
%
%Usage [x,xPert,nrv]= StandardGillespieSplit(tend,xin,xPertin, parameters,parametersPerturbed)
%
%
%               tend = end time. Real.
%               xin = initial condition for nominal process.  1 x 1 vector.
%               xPertin = initial condition for perturbed process.  1 x 1 vector.
%               
%               parameters = the parameters of the model.
%               Vector.  Only use 5 - 10
%
%               parametersPertubed = the parameters of the perturbed model.
%               Vector.  Only use 5 - 10
%
%
%The outpus are the following:
%
%                     x = output vector (final state) for nominal process.
%                     xPert = output vector (final state) for perturbed process.
%
%                     nrv = number of random variables utilized.
%
%Written by David F. Anderson on June 26th, 2024.

function [x,xPert,nrv]= StandardGillespieSplit(tend,xin,xPertin, parameters,parametersPerturbed)

%SPLIT COUPLING.

%max number of steps
N=10^3;

kb=parameters(5);
kd=parameters(6);

kbp = parametersPerturbed(5);
kdp = parametersPerturbed(6);

%Rates for the (split) coupled processes (3 for each process)
lambda = zeros([2,3]);

%Placeholders for sums of lambda(i,:) (so can use Gillespie implementation)
A = zeros([2,1]);

%Rates for nominal process
a = zeros([2,1]);

%Rates for perturbed process
b = zeros([2,1]);

%Initialize
x = xin;
xPert = xPertin;

times=zeros([1,N+1]);
nrv=0;

%main loop
for i = 1:N
    
    %intensities for nominal process
    a(1) = kb;
    a(2) = kd*x(1);
    
    %intensities for perturbed process.
    b(1) = kbp;
    b(2) = kdp*xPert(1);
    
    %intensities for coupled process.  First is min, then extra for
    %nominal, then extra for perturbed.  Finally, A(k) gives sum (max)
    for k = 1:2
        lambda(k,1) = min(a(k),b(k));
        lambda(k,2) = a(k) - lambda(k,1);
        lambda(k,3) = b(k) - lambda(k,1);
        A(k) = lambda(k,1)+lambda(k,2)+lambda(k,3);
    end
    
    %Finding total intensities. 
    lambda0 = 0;
    for k = 1:2
        lambda0 = lambda0 + A(k);
    end
    
    r = rand;
    nrv=nrv+1;
    
    %Time until next reaction
    Delta = log(1/r)/lambda0;
    
    times(1,i+1)=times(1,i)+Delta;
    if times(1,i+1)>tend
        break
    else
        
        r2=rand;
        nrv=nrv+1;
             
        %Find the overall reaction that happened.  Then discover which one.
        
        if r2<A(1)/lambda0 %reaction 1, 0 --> S
            if r2 < lambda(1,1)/lambda0 %update both
                x(1) = x(1) + 1;
                xPert(1) = xPert(1) + 1;
            elseif r2 < (lambda(1,1) + lambda(1,2))/lambda0 %update nominal
                x(1) = x(1) + 1;
            else %update perturbed
                xPert(1) = xPert(1) + 1;
            end
                        
        else           %reaction 2, S --> 0
            if r2 < (A(1) + lambda(2,1))/lambda0 %update both
                x(1) = x(1) - 1;
                xPert(1) = xPert(1) - 1;
            elseif r2 < (A(1) + lambda(2,1) + lambda(2,2))/lambda0 %update nominal
                x(1) = x(1) - 1;
            else %update perturbed
                xPert(1) = xPert(1) - 1;
            end
        end
            
    end
    
end

if i >= N
    55555
    'Too big in Gillespie'
end
