%The code produces a single realization of coupled paths for the following RNIC model.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 
%
%              with KI = 1, KE = 0.01.  So that KF = 0 and KC = 0.
%
%                    Chemistry:  0 -[kb]-> S,
%                                S -[kd]-> 0,
%                                
%
%          with rates kb = 1 and kd = 0.1.  
%          These are the rates from the paper by Duso and Zechner. 
%
%           Another important parameter is the mean of the input
%           distribution, which is Poisson(lambda).  We take:
%
%           lambda = 10;
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_Split(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               MCInitPert = the starting number of compartments for the perturbed process.  Integer.
%               Xinit = vector of size 1 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               XinitPert = vector of size 1 x (CompartmentMax) giving initial
%                       condition (with MCInit) for perturbed process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               parametersPerturbed = the parameters of the perturbed model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     1 x CompartmentMax 
%
%                     MCPert = number giving the ending size of the compartment count for perturbed process
%
%                     XPert = state of the perturbed system at end time.  Vector of size
%                     1 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%We simulate the compartment events using the split coupling.  Then call
%for a split coupling of the chemical systems between compartment events.
%
%Written by David F. Anderson, June 26th, 2024.

function [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_SplitNEW(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%This is for a perturbation in the compartment model.  Trying KE at first.

%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^3;


%State of the system.
X = Xinit;
MC = MCInit;
XPert = XinitPert;
MCPert = MCInitPert;

%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);

KIPert = parametersPerturbed(1);
KEPert = parametersPerturbed(2);
KFPert = parametersPerturbed(3);
KCPert = parametersPerturbed(4);

lambdaInput = parameters(7);
lambdaInputPerturbed = parametersPerturbed(7);


%Rates for the nominal compartments.
a = zeros([4,1]);
%Rates for the perturbed compartments
b = zeros([4,1]);

%Rates I need for the split coupling.
lambda = zeros([4,3]);
A = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input (nominal process)
    a(1) = KI;
    %Rate of compartment exit (nominal process)
    a(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C.  (nominal process)
    a(3) = KF*MC;
    %Rate of Coagulation (nominal process)
    a(4) = KC*MC*(MC-1);
    
    %Rate of compartment input
    b(1) = KIPert;
    %Rate of compartment exit
    b(2) = KEPert*MCPert;
    %Rate of fragmentation, C -[KF]-> 2C. 
    b(3) = KFPert*MCPert;
    %Rate of Coagulation
    b(4) = KCPert*MCPert*(MCPert-1);

    %Needed rates for the split
    for k = 1:4
        lambda(k,1) = min(a(k),b(k));  %min
        lambda(k,2) = a(k) - lambda(k,1);  %nominal excess
        lambda(k,3) = b(k) - lambda(k,1);  %perturbed excess
        A(k) = lambda(k,1)+lambda(k,2)+lambda(k,3);
    end
    
    %lambda0 will be absolute total rate
    lambda0 = 0;
    for k = 1:4
        for ell = 1:3
            lambda0 = lambda0+lambda(k,ell);
        end
    end
    
    %Get time of next compartment event.  Standard method.
    r1 = rand;
    NRV=NRV+1;

    Delta = log(1/r1)/lambda0;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.

    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %I NEED TO COME BACK TO THIS!!!!!!!  THIS IS THE GENERATION OF PATHS. 
    %At this point, we have MC and MCPert compartments.  Find the minimum
    %and generate that many paths.
    
    %Note that if I set minOfMC = 0, then the chemistry will simply update
    %independently.  I can turn this off in that case, which is nice to see
    %the effect it has.
    
    minOfMC = min(MC,MCPert);

    for k = 1:minOfMC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),X(:,k),XPert(:,k),parameters,parametersPerturbed);
        X(:,k) = x;
        XPert(:,k) = xPert;
        NRV=NRV+nrvs;
    end
    
    %update extra unperturbed
    for k = (minOfMC+1):MC
        [x,nrvs] = StandardGillespie(times(i+1)-times(i),X(:,k),parameters);
        X(:,k) = x;
        NRV = NRV+nrvs;
    end
    %update for the perturbed
    for k = (minOfMC+1):MCPert
        [x,nrvs] = StandardGillespie(times(i+1)-times(i),XPert(:,k),parametersPerturbed);
        XPert(:,k) = x;
        NRV = NRV+nrvs;
    end

    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %We are done if this happens.
        break
        
    else

        r2=rand;
        NRV = NRV+1;

        if r2 < A(1)/lambda0 %reaction 1, 0 --> C, appearance of compartment at end

            if r2 < lambda(1,1)/lambda0 %update both
               
                %New compartment. 
                if lambdaInput <= lambdaInputPerturbed
                    diff=lambdaInputPerturbed-lambdaInput;
                    
                    InputValue = poissrnd(lambdaInput);
                    NRV = NRV+1;
                    InputValuePert = InputValue + poissrnd(diff);
                    NRV = NRV+1;
                    
                else
                    diff=lambdaInput-lambdaInputPerturbed;
                    
                    InputValuePert = poissrnd(lambdaInputPerturbed);
                    NRV=NRV+1;
                    InputValue = InputValuePert+poissrnd(diff);
                    NRV=NRV+1;
                end

                
                X(:,MC+1) = InputValue;
                XPert(:,MCPert+1) = InputValuePert;
                
                %Update number of compartments
                MC = MC+1;
                MCPert = MCPert+1;
                
            elseif r2 < (lambda(1,1)+lambda(1,2))/lambda0 %update unperturbed
 
                %New compartment.
                InputValue = poissrnd(lambdaInput);
                NRV = NRV+1;
                
                X(:,MC+1) = InputValue;
                
                %Update number of compartments
                MC = MC+1;
                
            else %update perturbed
                %New compartment
                InputValuePert = poissrnd(lambdaInputPerturbed);
                NRV = NRV+1;
                XPert(:,MC+1) = InputValuePert;
                
                %Update number of compartments
                MCPert = MCPert+1;
            end
                

        elseif r2 < (A(1) + A(2))/lambda0 %reaction 2, C --> 0, deletion
            
            if r2 <(A(1) + lambda(2,1))/lambda0 %update both
                
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted.  Used the same random
                %variable, ufind, to locate it.  Mention in paper?
                IndexDel = floor(ufind*MC)+1;
                IndexDelPert = floor(ufind*MCPert)+1;
                
      
                %update all before this index
                for ell = 1:IndexDel-1
                    X(:,ell) = X(:,ell);
                end
                for ell = 1:IndexDelPert-1
                    XPert(:,ell) = XPert(:,ell);
                end
                
                %Now shift everything down
                for ell = IndexDel:MC-1
                    X(:,ell) = X(:,ell+1);
                end
                %Now shift everything down
                for ell = IndexDelPert:MCPert-1
                    XPert(:,ell) = XPert(:,ell+1);
                end
                %Update number of compartments
                MC = MC-1;
                MCPert = MCPert-1;
                
            elseif r2 < (A(1) + lambda(2,1) + lambda(2,2))/lambda0 %update unperturbed
                       
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted
                IndexDel = floor(ufind*MC)+1;                
      
                %update all before this index
                for ell = 1:IndexDel-1
                    X(:,ell) = X(:,ell);
                end
                
                %Now shift everything down
                for ell = IndexDel:MC-1
                    X(:,ell) = X(:,ell+1);
                end

                %Update number of compartments
                MC = MC-1;
                
            else %update perturbed
                            
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted
                IndexDelPert = floor(ufind*MCPert)+1;
                
                for ell = 1:IndexDelPert-1
                    XPert(:,ell) = XPert(:,ell);
                end
                
                %Now shift everything down
                for ell = IndexDelPert:MCPert-1
                    XPert(:,ell) = XPert(:,ell+1);
                end
                %Update number of compartments
                MCPert = MCPert-1;
                
            end


        elseif r2<(A(1)+A(2)+A(3))/lambda0 %reaction 3, C --> 2C, fragmentation.

            1111
            'Fragmentation?'
            if r2<(A(1) + A(2) + lambda(3,1))/lambda0  %update both
                
                %Pick which compartment fragments.  Note state dependent, so easy.
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted
                IndexFrag = floor(ufind*MC)+1;
                IndexFragPert = floor(ufind*MCPert)+1;
                
                %Get the Ss remaining
                %Take the minimum one to be binomial and have them share.
                %then add another, independent binomial to the other.
                if X(1,IndexFrag) > XPert(1,IndexFragPert)
                    SIndexPert = binornd(XPert(1,IndexFragPert),1/2);
                    SIndex = SIndexPert + binornd(X(1,IndexFrag)-XPert(1,IndexFragPert),1/2);
                else
                    SIndex = binornd(X(1,IndexFrag),1/2);
                    SIndexPert = SIndex + binornd(XPert(1,IndexFragPert)-X(1,IndexFrag),1/2);
                end
                NRV=NRV+2;                

                 
                
                %Add new compartment and update new compartment
                X(1,MC+1) = X(1,IndexFrag)-SIndex;
                
                XPert(1,MCPert+1) = XPert(1,IndexFragPert)-SIndexPert;
                
                %update IdexFrag
                X(1,IndexFrag) = SIndex;
                
                XPert(1,IndexFragPert) = SIndexPert;
                
                MC = MC+1;
                MCPert = MCPert+1;
                
            elseif r2<(A(1) + A(2) + lambda(3,1)+lambda(3,2))/lambda0  %update unperturbed
                     
                %Pick which compartment fragments.  Not state dependent, so easy.
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted
                IndexFrag = floor(ufind*MC)+1;

                %Get the Ss remaining
                SIndex = binornd(X(1,IndexFrag),1/2);
                NRV=NRV+1;
                
                %Add new compartment and update new compartment
                X(1,MC+1) = X(1,IndexFrag)-SIndex;
                
                %update IdexFrag
                X(1,IndexFrag) = SIndex;
                
                MC = MC+1;
                
            else %update perturbed
                            
                %Pick which compartment fragments.  Not state dependent, so easy.
                ufind = rand;
                NRV = NRV+1;
                
                %Which compartments to be deleted
                IndexFragPert = floor(ufind*MCPert)+1;
                                
                %Get the Ss remaining
                SIndexPert = binornd(XPert(1,IndexFragPert),1/2);
                NRV=NRV+1;
                
                %Add new compartment and update new compartment
                XPert(1,MCPert+1) = XPert(1,IndexFragPert)-SIndexPert;
                
                %update IdexFrag
                XPert(1,IndexFragPert) = SIndexPert;
                
                MCPert = MCPert+1;
                
            end

        else %reaction 4, 2C --> C, coagulation

            22222
            'Coagulation?'
            if r2<(A(1)+A(2)+A(3) + lambda(4,1))/lambda0 %update both
                
                %select two indices at random
                U1 = rand;
                ri1 = floor(U1*MC)+1;
                ri1Pert = floor(U1*MCPert)+1;
                NRV = NRV+1;
                
                %Now I need the second index.  From the set {1,...,MC}\{ri1}
                U2 = rand;
                NRV=NRV+1;
                ri2 = floor(U2*(MC-1))+1;
                %that works if ri2<ri1.  If ri2 >= ri1, add one.
                if ri2 >= ri1
                    ri2 = ri2+1;
                end
                
                ri2Pert = floor(U2*(MCPert-1))+1;
                %that works if ri2Pert<ri1Pert.  If ri2Pert >= ri1Pert, add one.
                if ri2Pert >= ri1Pert
                    ri2Pert = ri2Pert+1;
                end
                
                %I will put the material in ri1;
                             
                %collect in ri1 compartment.
                X(:,ri1)=X(:,ri1)+X(:,ri2);
                XPert(:,ri1Pert) = XPert(:,ri1Pert)+XPert(:,ri2Pert);
                
                %Now I have to delete ri2 and shift everything down.
                for k = ri2:MC-1
                    X(:,k) = X(:,k+1);
                end
                for k = ri2Pert:MCPert-1
                    XPert(:,k) = XPert(:,k+1);
                end
                
                %update number of compartments.
                MC = MC-1;
                MCPert = MCPert-1;
                
            elseif r2<(A(1)+A(2)+A(3) + lambda(4,1)+lambda(4,2))/lambda0 %update unperturbed
                
                %select two indices at random
                U1 = rand;
                ri1 = floor(U1*MC)+1;
                NRV = NRV+1;
                
                %Now I need the second index.  From the set {1,...,MC}\{ri1}
                U2 = rand;
                NRV=NRV+1;
                ri2 = floor(U2*(MC-1))+1;
                %that works if ri2<ri1.  If ri2 >= ri1, add one.
                if ri2 >= ri1
                    ri2 = ri2+1;
                end
                
                %I will put the material in ri1;
                             
                %collect in ri1 compartment.
                X(:,ri1)=X(:,ri1)+X(:,ri2);
                
                %Now I have to delete ri2 and shift everything down.
                for k = ri2:MC-1
                    X(:,k) = X(:,k+1);
                end
                
                %update number of compartments.
                MC = MC-1;
                
            else %update perturbed
                
                %select two indices at random
                U1 = rand;
                ri1Pert = floor(U1*MCPert)+1;
                NRV = NRV+1;
                
                %Now I need the second index.  From the set {1,...,MC}\{ri1}
                U2 = rand;
                NRV=NRV+1;
                
                ri2Pert = floor(U2*(MCPert-1))+1;
                %that works if ri2Pert<ri1Pert.  If ri2Pert >= ri1Pert, add one.
                if ri2Pert >= ri1Pert
                    ri2Pert = ri2Pert+1;
                end
                
                %I will put the material in ri1;
                             
                %collect in ri1 compartment.
                XPert(:,ri1Pert) = XPert(:,ri1Pert)+XPert(:,ri2Pert);
                
                %Now I have to delete ri2 and shift everything down.
                for k = ri2Pert:MCPert-1
                    XPert(:,k) = XPert(:,k+1);
                end
                
                %update number of compartments.
                MCPert = MCPert-1;
                
            end
            
        end

    end
end


if i >=NMax
    4444
    'over NMax'
end


