%The code produces a single realization of the following chemical model.

%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [x,nrv]= StandardGillespie(tend,xin,parameters,RVs)
%
%               parameters = the parameters of the model.
%                            It is a vector, but only use  the 5th - 10th
%                            elements (the others are for the compartment)
%               tend = end time. Real.
%               xin = initial condition.  3 x 1vector.  mRNA, Protein, Dimer
%               RVs = inputed uniform random variables.
%
%
%The outpus are the following:
%
%                     x = output vector (final state)
%
%                     nrv = number of random variables utilized.
%
%
%
%Written by David F. Anderson, June 4th, 2024.

function [x,nrv]= StandardGillespieWithRVs(tend,xin,parameters,RVs)

%track the random variables used
RVCount = 1;

%max number of steps
N = size(RVs);
N=N(1);

k1 = parameters(5);
k2 = parameters(6);
k3 = parameters(7);
dM = parameters(8);
dP = parameters(9);
dD = parameters(10);

x = xin;
times=zeros([1,N+1]);
nrv=0;

for i = 1:N
    
    a1 = k1;
    a2 = dM*x(1);
    a3 = k2*x(1);
    a4 = dP*x(2);
    a5 = k3*x(2)*(x(2)-1);
    a6 = dD*x(3);
    
    a0 = a1+a2+a3+a4+a5+a6;
    
    %Use the proper RVs.  nrv is superfluous, but will keep it for
    %consistency with other codes.
    r = RVs(RVCount);
    RVCount = RVCount + 1;
    nrv=nrv+1;
    
    Delta = log(1/r)/a0;
    
    times(1,i+1)=times(1,i)+Delta;
    if times(1,i+1)>tend
        break
    else

        %Use the proper RV and update.
        r2=RVs(RVCount);
        RVCount=RVCount+1;
        
        nrv=nrv+1;

        if r2<a1/a0
            x(1) = x(1) + 1;

        elseif r2<(a1+a2)/a0
            x(1) = x(1) - 1;

        elseif r2<(a1+a2+a3)/a0
            x(2) = x(2)+1;

        elseif r2<(a1+a2+a3+a4)/a0
            x(2) = x(2) - 1;
        
        elseif r2<(a1+a2+a3+a4+a5)/a0
            x(2) = x(2) - 2;
            x(3) = x(3) + 1;
        else
            x(3) = x(3) - 1;
        end
    end
    
end

if i >= N
    'N problem'
end