%The code produces a single realization of two paths.  They are coupled through the compartments, 
%and the paths within the compartments are coupled via the common reaction
%path (CRP) method.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_CRP(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               MCInitPert = the starting number of compartments for the perturbed process.  Integer.
%               Xinit = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               XinitPert = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for perturbed process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               parametersPerturbed = the parameters of the perturbed model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     MCPert = number giving the ending size of the compartment count for perturbed process
%
%                     XPert = state of the perturbed system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%We simulate the compartment events a single time so the processes are coupled completely in that way.  
%We then simulate the various compartments between those events.
%
%Written by David F. Anderson, June 4th, 2024.


function [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_CRP(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%THIS ONLY WORKS IF THE PERTURBED PARAMETER IS ONE OF THE CHEMISTRY
%ONES!!!!!!!

%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^3;


%State of the system.
X = Xinit;
MC = MCInit;
XPert = XinitPert;
MCPert = MCInitPert;


%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
% k1 = parameters(5);
% k2 = parameters(6);
% k3 = parameters(7);
% dM = parameters(8);
% dP = parameters(9);
% dD = parameters(10);


%Rates for the compartments.
lambdaComp = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input
    lambdaComp(1) = KI;
    %Rate of compartment exit
    lambdaComp(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C. 
    lambdaComp(3) = KF*MC;
    %Rate of Coagulation
    lambdaComp(4) = KC*MC*(MC-1);

    %Total rate for compartments
    TotalRate = lambdaComp(1)+lambdaComp(2)+lambdaComp(3)+lambdaComp(4);
    
    %Get time of next compartment event.  Standard method.
    r1 = rand;
    NRV=NRV+1;

    Delta = log(1/r1)/TotalRate;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.
    %If so, we still need to simulate up to the end time.
    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %At this point, we have MC compartments.  Now we need to generate our two 
    %Independent Gillespie paths.  This can
    %be modified to use CRP, split coupling, or the common reaction path
    %method quite easily.  THIS IS THE ONLY THING THAT NEED TO CHANGE FOR
    %ANY OF THESE METHODS.
    for k = 1:MC
        %I need to send the RVs.  This should be plenty.
        RVs = rand([6,35]);
        RVs = log(1./RVs);
        [x,nrv1] = NRM_CRP(times(i+1)-times(i),X(:,k),parameters,RVs);
        %rng(seed);
        
        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPert(:,k),parametersPerturbed,RVs);
        %[x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),X(:,k),XPert(:,k),parameters,parametersPerturbed);
        X(:,k) = x;
        
        XPert(:,k) = xPert;
        NRV=NRV+ nrv1+nrv2;
    end

    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %We are done if this happens.
        break
    else
        
        %Not done.  See what happened with the compartments and update
        r2=rand;
        NRV = NRV+1;

        if r2 < lambdaComp(1)/TotalRate %appearance of compartment at end

            [X,XPert] = Entry(X,XPert,MC);
            
            %Update number of compartments
            MC = MC+1;

        elseif r2 < (lambdaComp(1)+lambdaComp(2))/TotalRate %deletion
            %find the one to be deleted.
            IndexDel = randi([1,MC]);
            NRV=NRV+1;

            [X,XPert] = Del(X,XPert,MC,IndexDel);
            
            %Update number of compartments
            MC = MC-1;


        elseif r2<(lambdaComp(1)+lambdaComp(2)+lambdaComp(3))/TotalRate %fragmentation.


            %Pick which compartment fragments.  Note state dependent, so easy.
            IndexFrag = randi([1,MC]);
            NRV=NRV+1;

            %There are 2*3 RVs used in the call to frag (2 per species).
            NRV = NRV+2*3;

            [X,XPert] = frag(X,XPert,MC,IndexFrag);

            MC = MC+1;

        else %Do coagulation

            %select two indices at random
            ri1=randi([1 MC]);
            ri2=randi([1 MC]);
            NRV=NRV+2;

            while ri1==ri2
                ri2=randi([1 MC]);
                NRV=NRV+1;
            end
            %I will put the material in ri1;

            [X,XPert] = coag(X,XPert,MC,ri1,ri2);

            %update number of compartments.
            MC = MC-1;

        end

    end
end

MCPert = MC;

if i >= NMax
    'NMax problem'
end

%
% MoutputPlot=zeros([number_timepoints+1,1]);
% PoutputPlot=zeros([number_timepoints+1,1]);
% DoutputPlot=zeros([number_timepoints+1,1]);
% for ell = 1:number_timepoints+1
%     MoutputPlot(ell) = Xoutput(1,1,ell);
%     PoutputPlot(ell) = Xoutput(2,1,ell);
%     DoutputPlot(ell) = Xoutput(3,1,ell);
% end
% figure
% plot(OutputTimes,MoutputPlot,'blue',OutputTimes, PoutputPlot,'green', OutputTimes,DoutputPlot,'red')
% legend('M','P','D');




    function [Xnew,XPertnew] = Entry(X,XPert,M)
        %All previous compartments are the same
        Xnew = X;
        XPertnew = XPert;
        %New compartment with nothing in it.  Might need to change this
        %sometime.
        Xnew(:,M+1) = zeros([3,1]);
        XPertnew(:,M+1) = zeros([3,1]);
    end

    function [Xnew,XPertnew] = Del(X,XPert,M,ID)
        Xnew=X;
        XPertnew=XPert;
        

        %Now shift everything down
        for ell = ID:M-1
            Xnew(:,ell) = X(:,ell+1);
            XPertnew(:,ell) = XPert(:,ell+1);
        end
    end

    function [Xnew,XPertnew] = frag(X,XPert,M,IF)

        Xnew = X;
        XPertnew = XPert;
        % %I want to couple these as tightly as possible.
        % if X(1,IF) > XPert(1,IF)
        %     MIndexPert = binornd(XPert(1,IF),1/2);
        %     MIndex = MIndexPert + binornd(X(1,IF)-XPert(1,IF),1/2);
        % else
        %     MIndex = binornd(X(1,IF),1/2);
        %     MIndexPert = MIndex + binornd(XPert(1,IF)-X(1,IF),1/2);
        % end
        % 
        % if X(2,IF) > XPert(2,IF)
        %     PIndexPert = binornd(XPert(2,IF),1/2);
        %     PIndex = PIndexPert + binornd(X(2,IF)-XPert(2,IF),1/2);
        % else
        %     PIndex = binornd(X(2,IF),1/2);
        %     PIndexPert = PIndex + binornd(XPert(2,IF)-X(2,IF),1/2);
        % end
        % 
        % if X(3,IF) > XPert(3,IF)
        %     DIndexPert = binornd(XPert(3,IF),1/2);
        %     DIndex = DIndexPert + binornd(X(3,IF)-XPert(3,IF),1/2);
        % else
        %     DIndex = binornd(X(3,IF),1/2);
        %     DIndexPert = DIndex + binornd(XPert(3,IF)-X(3,IF),1/2);
        % end

        %This is simply testing what happens without this coupling. It
        %seems we get a 20 fold increase in the overall variance.  This is
        %a big part of the coupling.
        % MIndex = binornd(X(1,IF),1/2);
        % MIndexPert = binornd(XPert(1,IF),1/2);
        % 
        % PIndex = binornd(X(2,IF),1/2);
        % PIndexPert = binornd(XPert(2,IF),1/2);
        %
        % DIndex = binornd(X(3,IF),1/2);
        % DIndexPert = binornd(XPert(3,IF),1/2);

        %Now let's try the hypergeometric idea. Should be good coupling,
        %but very slow.
        MIndex = binornd(X(1,IF),1/2);
        if X(1,IF) <= XPert(1,IF)
            MIndexPert = MIndex + binornd(XPert(1,IF)-X(1,IF),1/2);
        else
            MIndexPert = hygernd(X(1,IF),MIndex,XPert(1,IF));
        end

        PIndex = binornd(X(2,IF),1/2);
        if X(2,IF) <= XPert(2,IF)
            PIndexPert = PIndex + binornd(XPert(2,IF)-X(2,IF),1/2);
        else
            PIndexPert = hygernd(X(2,IF),PIndex,XPert(2,IF));
        end

        DIndex = binornd(X(3,IF),1/2);
        if X(3,IF) <= XPert(3,IF)
            DIndexPert = DIndex + binornd(XPert(3,IF)-X(3,IF),1/2);
        else
            DIndexPert = hygernd(X(3,IF),DIndex,XPert(3,IF));
        end



        %Add new compartment and update new compartment
        Xnew(1,M+1) = X(1,IF)-MIndex;
        Xnew(2,M+1) = X(2,IF)-PIndex;
        Xnew(3,M+1) = X(3,IF)-DIndex;

        XPertnew(1,M+1) = XPert(1,IF)-MIndexPert;
        XPertnew(2,M+1) = XPert(2,IF)-PIndexPert;
        XPertnew(3,M+1) = XPert(3,IF)-DIndexPert;

        %update IdexFrag
        Xnew(1,IF) = MIndex;
        Xnew(2,IF) = PIndex;
        Xnew(3,IF) = DIndex;

        XPertnew(1,IF) = MIndexPert;
        XPertnew(2,IF) = PIndexPert;
        XPertnew(3,IF) = DIndexPert;
    end

    function [Xnew,XPertnew] = coag(X,XPert,M,r1,r2)

        Xnew = X;
        XPertnew = XPert;
        %First, update all to be the same
        % for ell = 1:M
        %     Xnew(:,ell) = X(:,ell);
        %     XPertnew(:,ell) = XPert(:,ell);
        % end

        %collect in r1 compartment.
        Xnew(:,r1)=Xnew(:,r1)+Xnew(:,r2);
        XPertnew(:,r1) = XPertnew(:,r1)+XPertnew(:,r2);

        %Now I have to delete ri2 and shift everything down.
        for ell = r2:M-1
            Xnew(:,ell) = Xnew(:,ell+1);
            XPertnew(:,ell) = XPertnew(:,ell+1);
        end
    end

end