%
%  main.m	This is the outer code to call for multiple realizations of the
%           compartment model using coupled compartments, and the common reaction path method.  
%           The model is the following
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%              with KI = 2, KE = 1, KF = 3, and KC = 1.
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%          with rates k1=200, k2 = 20, k3 = 0.01, dM = 25, dP = 1, dD = 1.
%          These are the rates from the book written by Anderson and Kurtz,
%          Example 2.4
%
%          This code is useful if the parameters of the chemical systems,
%          and not the compartments, are being perturbed, as we are
%          coupling the compartments.
%
%  usage:	mainCoupledCompCRP(n,tend,h)
%
%               n = number of times to run to get stats.
%
%               tend = time of simulation
%
%               h = size of difference
%
%  example:	mainCoupledCompCRP(10000,10,1/10)
%
%  The output will be the mean and estimator variance of the statistic of
%  choice. I plan for that statistic to be the total (across all cells)
%  number of dimers (but this is arbitrary).
%
%
%Written by David F. Anderson, June 4th, 2024.

function [] = mainCoupledCompCRP(n,tend,h)

%For timing purposes
tic

%Set the maximum number of compartments (for pre-allocation).  This will be
%case specific.  For the model I'm using we should never even come close to
%this number of compartments.
CompartmentMax = 200;

%Set the number of timepoints we want (for plotting variances).  Will plot
%a total of number_timepoints+1.
number_timepoints = 200;
%This is the time frame increment
del = tend/number_timepoints;
%Placeholders for the different MC (number of compartments)
OutputMC1 = zeros([number_timepoints+1,1]);
OutputMC2 = zeros([number_timepoints+1,1]);

%The state of the system is now a 3 dimensional array.  It will be  size
%3 x CompartmentMax x number_timepoints+1,
%with the three giving each species, M, P, D, the CompartmentMax giving the
%compartment number (but we only care about it up to MC) and
%number_timepoints+1 giving time. 
Xnominal = zeros([3, CompartmentMax, number_timepoints+1]);
Xperturbed = zeros([3, CompartmentMax, number_timepoints+1]);

%Set the parameters
%The first four are the compartment parameters.
KI = 10;
KE = 3.5;
KF = 3;
KC = .1;
%The next 6 are the chemical parameters.
k1 = 200;
k2 = 10;
k3 = 0.01;
dM = 25;
dP = 1;
dD = 1;

%If I want to fix the seed of the random number generator
%rng(1234)

%I need the total number of dimers for each of the n iterations (first
%dimension) and for each of the different time points (second dimension).
TotalDimersNominal = zeros([n,number_timepoints+1]);
TotalDimersPerturbed = zeros([n,number_timepoints+1]);

%For the number of compartments
TotalCompartments = zeros([n,number_timepoints+1]);

%The difference (divided by h) of the two realizations.
deriv = zeros([n,number_timepoints+1]);

%EstDeriv will be the actual estimated derivative at each timepoint.
EstDeriv = zeros([number_timepoints+1,1]);
%EstVar will give the estimated variance at each timepoint.
EstVar = zeros([number_timepoints+1,1]);
%I would like to plot the estimated number of total dimers
EstDimers = zeros([number_timepoints+1,1]);
%For plotting the mean number of compartments
EstCompartments = zeros([number_timepoints+1,1]);


%Count the number of RVs used. Not sure I even want this.
RV_Count = 0;

% parameters for each model (nominal and perturbed)
parameters = [KI, KE, KF, KC, k1, k2, k3, dM, dP, dD];
parametersPerturbed =  [KI, KE, KF, KC, k1, k2+h, k3, dM, dP, dD];

% main loop
for j = 1:n
    
    %The two key objects that must be returned by a single realization are
    %following:
    %
    % 1. The value of MC (number of compartments) at each of the relevant time points.
    %
    % 2. The state of the RNIC model at each of the relevant time points.
    % Specifically, this means giving the counts of each species in the
    % MC(t) compartments, where MC(t) is the number of compartments at time
    % t, for all relevant t.
    %
    %
    %Also, but slightly less important, is NRVs, the number of random variables used.
    %
    %This will call to get both realizations: perturbed and nominal.
    %I am going to initialize everything with MC = 1 and all states set to
    %0.  That's also what I will do if a new cell appears (easy to change)
    
    [OutputMC1, Xnominal,OutputMC2, Xperturbed, NRVs1] = SingleRealizationFullPath_Shell_CRP(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints);

    %We should have that OutpuMC1=OutputMC2.  If not something went wrong
    if OutputMC1 ~= OutputMC2
        'Something is wrong.  Should have OutputMC1=OutputMC2'
        break
    end
    %Just to see how far along we are with the script.
    if mod(j,n/10)==0
        toc
        j
    end
    
    %Now to process data.  Will get the total number of dimers at each
    %time.
    for t = 1:number_timepoints+1
        %sum over all the relevant compartments.
        TotalDimersNominal(j, t) = sum(Xnominal(3,1:OutputMC1(t),t));
    end
    for t = 1:number_timepoints+1
        %sum over all the relevant compartments.
        TotalDimersPerturbed(j, t) = sum(Xperturbed(3,1:OutputMC2(t),t));
    end
    for t = 1:number_timepoints+1
       deriv(j,t) =  (1/h)*(TotalDimersPerturbed(j, t)-TotalDimersNominal(j, t));
    end
    
    for t = 1:number_timepoints+1
        TotalCompartments(j,t) = OutputMC1(t);
    end


    RV_Count = RV_Count + NRVs1;
end

%Output the number of random variables used.
RV_Count

for t = 1:number_timepoints+1
    EstDeriv(t) = mean(deriv(:,t));
    EstVar(t) = var(deriv(:,t))/n;
    EstDimers(t) = mean(TotalDimersNominal(:,t));
    EstCompartments(t) = mean(TotalCompartments(:,t));
end

toc

Tp = zeros([number_timepoints+1,1]);
for m = 2:number_timepoints+1
    Tp(m) = Tp(m-1) + del;
end

%Just going to plot from the first compartment

%This is the state of the system from a single compartment.  Not sure
%what it gives us actually.  Just going to plot from the first compartment
Xnom1 = zeros([number_timepoints+1,1]);
Xnom2 = zeros([number_timepoints+1,1]);
Xnom3 = zeros([number_timepoints+1,1]);
for k = 1:number_timepoints+1
    Xnom1(k) = Xnominal(1,1,k);
    Xnom2(k) = Xnominal(2,1,k);
    Xnom3(k) = Xnominal(3,1,k);
end
figure
plot(Tp,Xnom1,'blue', Tp,Xnom2,'green', Tp,Xnom3,'red')
legend('Sample mRNA','Sample Protein','Sample Dimer')

%Now the variance plot
figure
plot(Tp,EstVar)
legend('Estimator Variance')

figure
plot(Tp, EstDeriv)
legend('Estimated Derivative of total Dimers')

figure 
plot(Tp, EstDimers)
legend('Estimated mean of total Dimers')

figure 
plot(Tp, EstCompartments)
legend('Estimated mean of total compartments')

figure 
plot(Tp, OutputMC1)
legend('Realization of number of compartments from one run')


%sumtime

% CFD_data = vdata;
% save_CFD = ['CFD' '.mat'];
% save(save_CFD,'CFD_data');

end %The program

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
