%The code produces a single realization of two paths.  They are coupled through the compartments, 
%but the paths within the compartments are generated independently.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_Ind(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               MCInitPert = the starting number of compartments for the perturbed process.  Integer.
%               Xinit = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               XinitPert = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for perturbed process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               parametersPerturbed = the parameters of the perturbed model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     MCPert = number giving the ending size of the compartment count for perturbed process
%
%                     XPert = state of the perturbed system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%We simulate the compartment events a single time so the processes are coupled completely in that way.  
%We then simulate the various compartments between those events.
%
%Written by David F. Anderson, June 4th, 2024.


function [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_Ind(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%THIS ONLY WORKS IF THE PERTURBED PARAMETER IS ONE OF THE CHEMISTRY
%ONES!!!!!!!

%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^3;


%State of the system.
X = Xinit;
MC = MCInit;
XPert = XinitPert;
MCPert = MCInitPert;


%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
% k1 = parameters(5);
% k2 = parameters(6);
% k3 = parameters(7);
% dM = parameters(8);
% dP = parameters(9);
% dD = parameters(10);


%Rates for the compartments.
lambdaComp = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input
    lambdaComp(1) = KI;
    %Rate of compartment exit
    lambdaComp(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C. 
    lambdaComp(3) = KF*MC;
    %Rate of Coagulation
    lambdaComp(4) = KC*MC*(MC-1);

    %Total rate for compartments
    TotalRate = lambdaComp(1)+lambdaComp(2)+lambdaComp(3)+lambdaComp(4);
    
    %Get time of next compartment event.  Standard method.
    r1 = rand;
    NRV=NRV+1;

    Delta = log(1/r1)/TotalRate;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.
    %If so, we still need to simulate up to the end time.
    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %At this point, we have MC compartments.  Now we need to generate our two 
    %Independent Gillespie paths.  This can
    %be modified to use CRN, split coupling, or the common reaction path
    %method quite easily.  THIS IS THE ONLY THING THAT NEED TO CHANGE FOR
    %ANY OF THESE METHODS.
    for k = 1:MC
        
        [x,nrv1] = StandardGillespie(times(i+1)-times(i),X(:,k),parameters);
        
        [xPert,nrv2] = StandardGillespie(times(i+1)-times(i),XPert(:,k),parametersPerturbed);    
        

        %[x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),X(:,k),XPert(:,k),parameters,parametersPerturbed);
        X(:,k) = x;
        XPert(:,k) = xPert;
        NRV=NRV+ nrv1+nrv2;
    end

    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %We are done if this happens.
        break
    else
        
        %Not done.  See what happened with the compartments and update
        r2=rand;
        NRV = NRV+1;

        if r2 < lambdaComp(1)/TotalRate %appearance of compartment at end

            %All previous compartments are the same
            X(:,1:MC) = X(:,1:MC);
            XPert(:,1:MC) = XPert(:,1:MC);

            %New compartment with nothing in it.  Might need to change this
            %sometime.
            
            X(:,MC+1) = zeros([3,1]);
            XPert(:,MC+1) = zeros([3,1]);
            
            %Update number of compartments
            MC = MC+1;

        elseif r2 < (lambdaComp(1)+lambdaComp(2))/TotalRate %deletion
            %find the one to be deleted.
            IndexDel = randi([1,MC]);
            NRV=NRV+1;
            
            %update all before this index
            for ell = 1:IndexDel-1
                X(:,ell) = X(:,ell);
                XPert(:,ell) = XPert(:,ell);
            end
            %Now shift everything down
            for ell = IndexDel:MC-1
                X(:,ell) = X(:,ell+1);
                XPert(:,ell) = XPert(:,ell+1);
            end
            %Update number of compartments
            MC = MC-1;


        elseif r2<(lambdaComp(1)+lambdaComp(2)+lambdaComp(3))/TotalRate %fragmentation.


            %Pick which compartment fragments.  Note state dependent, so easy.
            IndexFrag = randi([1,MC]);
            NRV=NRV+1;

            
            NRV = NRV+3*2;

            %I want to couple these as tightly as possible.
            if X(1,IndexFrag) > XPert(1,IndexFrag)
                MIndexPert = binornd(XPert(1,IndexFrag),1/2);
                MIndex = MIndexPert + binornd(X(1,IndexFrag)-XPert(1,IndexFrag),1/2);
            else
                MIndex = binornd(X(1,IndexFrag),1/2);
                MIndexPert = MIndex + binornd(XPert(1,IndexFrag)-X(1,IndexFrag),1/2);
            end

            if X(2,IndexFrag) > XPert(2,IndexFrag)
                PIndexPert = binornd(XPert(2,IndexFrag),1/2);
                PIndex = PIndexPert + binornd(X(2,IndexFrag)-XPert(2,IndexFrag),1/2);
            else
                PIndex = binornd(X(2,IndexFrag),1/2);
                PIndexPert = PIndex + binornd(XPert(2,IndexFrag)-X(2,IndexFrag),1/2);
            end

            if X(3,IndexFrag) > XPert(3,IndexFrag)
                DIndexPert = binornd(XPert(3,IndexFrag),1/2);
                DIndex = DIndexPert + binornd(X(3,IndexFrag)-XPert(3,IndexFrag),1/2);
            else
                DIndex = binornd(X(3,IndexFrag),1/2);
                DIndexPert = DIndex + binornd(XPert(3,IndexFrag)-X(3,IndexFrag),1/2);
            end

            %update all compartments to be exactly the same
            for k = 1:MC
                X(:,k) = X(:,k);
                XPert(:,k) = XPert(:,k);
            end

            %Add new compartment and update new compartment
            X(1,MC+1) = X(1,IndexFrag)-MIndex;
            X(2,MC+1) = X(2,IndexFrag)-PIndex;
            X(3,MC+1) = X(3,IndexFrag)-DIndex;
            
            XPert(1,MC+1) = XPert(1,IndexFrag)-MIndexPert;
            XPert(2,MC+1) = XPert(2,IndexFrag)-PIndexPert;
            XPert(3,MC+1) = XPert(3,IndexFrag)-DIndexPert;

            %update IdexFrag
            X(1,IndexFrag) = MIndex;
            X(2,IndexFrag) = PIndex;
            X(3,IndexFrag) = DIndex;
            
            XPert(1,IndexFrag) = MIndexPert;
            XPert(2,IndexFrag) = PIndexPert;
            XPert(3,IndexFrag) = DIndexPert;

            MC = MC+1;

        else %Do coagulation

            %select two indices at random
            ri1=randi([1 MC]);
            ri2=randi([1 MC]);
            NRV=NRV+2;

            while ri1==ri2
                ri2=randi([1 MC]);
                NRV=NRV+1;
            end
            %I will put the material in ri1;

            %First, update all to be the same
            for k = 1:MC
                X(:,k) = X(:,k);
                XPert(:,k) = XPert(:,k);
            end

            %collect in ri1 compartment.
            X(:,ri1)=X(:,ri1)+X(:,ri2);
            XPert(:,ri1) = XPert(:,ri1)+XPert(:,ri2);
            
            %Now I have to delete ri2 and shift everything down.
            for k = ri2:MC-1
                X(:,k) = X(:,k+1);
                XPert(:,k) = XPert(:,k+1);
            end

            %update number of compartments.
            MC = MC-1;



        end

    end
end

MCPert = MC;

if i >= NMax
    'NMax problem'
end

% 
% MoutputPlot=zeros([number_timepoints+1,1]);
% PoutputPlot=zeros([number_timepoints+1,1]);
% DoutputPlot=zeros([number_timepoints+1,1]);
% for ell = 1:number_timepoints+1
%     MoutputPlot(ell) = Xoutput(1,1,ell);
%     PoutputPlot(ell) = Xoutput(2,1,ell);
%     DoutputPlot(ell) = Xoutput(3,1,ell);
% end
% figure
% plot(OutputTimes,MoutputPlot,'blue',OutputTimes, PoutputPlot,'green', OutputTimes,DoutputPlot,'red')
% legend('M','P','D');


