%The code produces a single realization of the following chemical model.
%The key point is that six streams of unit-exponentials are provided and 
%we use those to make the Poisson processes.  We do this so we can
%implement the common reaction path method.
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [x,nrv]= NRM_CRP(tend,xin,parameters,RVs)
%
%               parameters = the parameters of the model.
%                            It is a vector, but only use  the 5th - 10th
%                            elements (the others are for the compartment)
%               tend = end time. Real.
%               xin = initial condition.  3 x 1vector.  mRNA, Protein, Dimer
%               RVs = inputed unit exponential random variables.  This will
%                     be an array of size 6 x number_exponentials
%
%
%The outpus are the following:
%
%                     x = output vector (final state)
%
%                     nrv = number of random variables utilized.
%
%
%
%Written by David F. Anderson, June 4th, 2024.

function [x,nrv]= NRM_CRP(tend,xin,parameters,RVs)

%track the random variables used
RVCount = 1;

%max number of steps
N=50;

k1 = parameters(5);
k2 = parameters(6);
k3 = parameters(7);
dM = parameters(8);
dP = parameters(9);
dD = parameters(10);

%Initialize intensities.
lambda = zeros([6,1]);

x = xin;
times=zeros([1,N+1]);
nrv=0;

%Internal times of the integrated intensity functions.
T= zeros([6,1]);

%Times for the next unit Poisson process in the stream.
%Pointer(k) will tell us where each stream should take NEXT.
Pointer = ones([6,1]);
P= zeros([6,1]);
for k = 1:6
    P(k) = RVs(k,Pointer(k));
    Pointer(k) = Pointer(k)+1;
    nrv=nrv+1;
end

%Initialize Deltas (internal time until next reaction)
IntensityDelta = zeros([6,1]);
for i = 1:N
    
    lambda(1) = k1;
    lambda(2) = dM*x(1);
    lambda(3) = k2*x(1);
    lambda(4) = dP*x(2);
    lambda(5) = k3*x(2)*(x(2)-1);
    lambda(6) = dD*x(3);
    
    %I could do this as vectors.  Could change later if I want.    
    for k = 1:6
        if lambda(k)>0
            IntensityDelta(k) = (P(k) - T(k))/lambda(k);
        else
            IntensityDelta(k) = Inf;
        end
    end

    %Find the minimum and its index.
    if IntensityDelta(1) < IntensityDelta(2) && IntensityDelta(1) < IntensityDelta(3) && IntensityDelta(1) < IntensityDelta(4) && IntensityDelta(1) < IntensityDelta(5) && IntensityDelta(1) < IntensityDelta(6)
        minDelta = IntensityDelta(1);
        indexMin = 1;
    elseif IntensityDelta(2) < IntensityDelta(3) && IntensityDelta(2) < IntensityDelta(4) && IntensityDelta(2) < IntensityDelta(5) && IntensityDelta(2) < IntensityDelta(6)
        minDelta = IntensityDelta(2);
        indexMin = 2;
    elseif IntensityDelta(3) < IntensityDelta(4) && IntensityDelta(3) < IntensityDelta(5) && IntensityDelta(3) < IntensityDelta(6)
        minDelta = IntensityDelta(3);
        indexMin = 3;
    elseif IntensityDelta(4) < IntensityDelta(5) && IntensityDelta(4) < IntensityDelta(6)
        minDelta = IntensityDelta(4);
        indexMin = 4;
    elseif IntensityDelta(5) < IntensityDelta(6)
        minDelta = IntensityDelta(5);
        indexMin = 5;
    else
        minDelta = IntensityDelta(6);
        indexMin = 6;
    end
        
   % [minDelta, indexMin] = min(IntensityDelta);


    %update time.
    times(1,i+1)=times(1,i)+minDelta;
    
    %If the next reaction happens after tend, we are done.
    if times(1,i+1)>tend
        break
    
        %Otherwise, find which reaction happened and update.
    else

        %Update all the integrated intensities.
        for k = 1:6
            T(k) = T(k) + lambda(k)*minDelta;
        end
        
        %Only one reaction needs a new unit-exponential.  Do that and add
        %to it's pointer.  
        P(indexMin) = P(indexMin)+RVs(indexMin,Pointer(indexMin));
        
        Pointer(indexMin) = Pointer(indexMin)+1;
        
        nrv = nrv+1;
        %Update the reaction.  (Would be nice to have reaction vectors)
        
        if indexMin == 1
            x(1) = x(1) + 1;

        elseif indexMin == 2
            x(1) = x(1) - 1;

        elseif indexMin == 3
            x(2) = x(2) + 1;

        elseif indexMin == 4
            x(2) = x(2) - 1;
        
        elseif indexMin == 5
            x(2) = x(2) - 2;
            x(3) = x(3) + 1;
        else
            x(3) = x(3) - 1;
        end
    end
    
end