%The code produces a single realization of two paths.  They are coupled through the compartments, 
%and the paths within the compartments are coupled via the common reaction
%path method (CRP)
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MCoutput,Xoutput,MCoutputPert,XoutputPert,NRV]= SingleRealizationFullPath_Shell_CRP(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints)
%
%
%               parametersPerturbed = the perturbed parameters of the model (listed above). Vector.
%               parameters = the nominal parameters of the model (listed above). Vector.
%               tend = end time. Real.
%               CompartmentMax = maximum number of compartments.  Integer.
%               number_timepoints = total number of timepoints for the purposes of plotting.  Integer.
%
%
%The outpus are the following:
%
%                     MCoutput = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time points.
%
%                     Xoutput = state of system at relevant times for nominal model.  Vector of size
%                     3 x CompartmentMax x (number_timepoints + 1)
%
%                     MCoutputPert = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time
%                     points for the perturbed model.
%
%                     XoutputPert = state of system at relevant times for perturbed model.  Vector of size
%                     3 x CompartmentMax x (number_timepoints + 1)
%
%                     NRV = number of random variables utilized.
%
%Written by David F. Anderson, Nobember 24th, 2024.


function [MCoutput, Xnom, Xpertk1, Xpertk2, Xpertk3, XpertdM, XpertdP, XpertdD, NRV] = SingleRealizationFullPath_Shell_CRP(PertFactor, parameters, tend, CompartmentMax, number_timepoints)

%Need to have the times that I will be updating my outputs. This is the
%amount of time I must simulate.
del = tend/number_timepoints;
OutputTimes = zeros([number_timepoints+1,1]);
for ell = 1:number_timepoints
    OutputTimes(ell+1) = OutputTimes(ell) + del;
end

%State of the system at relevant time-points.
Xnom = zeros([3,CompartmentMax,number_timepoints+1]);
Xpertk1 = zeros([3,CompartmentMax,number_timepoints+1]);
Xpertk2 = zeros([3, CompartmentMax, number_timepoints+1]);
Xpertk3 = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdM = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdP = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdD = zeros([3, CompartmentMax, number_timepoints+1]);

MCoutput = zeros([number_timepoints+1,1]);

%INITIALIZE
MCoutput(1)=1;

Xinitial=zeros([3,1]);
XPertInitial=zeros([3,1]);

%initialize all the processes

for k = 1:MCoutput(1)
    Xnom(:,k,1) = Xinitial;
    Xpertk1(:,k,1) = XPertInitial;
    Xpertk2(:,k,1) = XPertInitial;
    Xpertk3(:,k,1) = XPertInitial;
    XpertdM(:,k,1) = XPertInitial;
    XpertdP(:,k,1) = XPertInitial;
    XpertdD(:,k,1) = XPertInitial;
end

%How many random variables used
NRV=0;

%Main Loop to get the coupled paths.
for ell = 1:number_timepoints
    %MCoutput_new should be a single number
    %Xoutput_newk1, etc., should be a vector of size 3 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %XoutputPert_newk1, etc., should be a vector of size 3 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %NRV_new is the number of random variables used in this single step (of
    %size del)
    [MCoutput_new, Xoutput_new, XoutputPert_newk1, XoutputPert_newk2, XoutputPert_newk3, XoutputPert_newdM, XoutputPert_newdP, XoutputPert_newdD, NRV_new] = SingleRealizationSingleTimeCompShell_CRP(MCoutput(ell), Xnom(:,:,ell), Xpertk1(:,:,ell), Xpertk2(:,:,ell), Xpertk3(:,:,ell), XpertdM(:,:,ell), XpertdP(:,:,ell), XpertdD(:,:,ell), parameters, PertFactor, del);

    MCoutput(ell+1) = MCoutput_new;
    
    for k = 1:MCoutput_new
        Xnom(:,k,ell+1) = Xoutput_new(:,k);
        Xpertk1(:,k,ell+1) = XoutputPert_newk1(:,k);
        Xpertk2(:,k,ell+1) = XoutputPert_newk2(:,k);
        Xpertk3(:,k,ell+1) = XoutputPert_newk3(:,k);
        XpertdM(:,k,ell+1) = XoutputPert_newdM(:,k);
        XpertdP(:,k,ell+1) = XoutputPert_newdP(:,k);
        XpertdD(:,k,ell+1) = XoutputPert_newdD(:,k);
    end
    NRV = NRV + NRV_new;
end


