%The code produces a single realization of two paths.  They are coupled through the compartments, 
%and the paths within the compartments are coupled via the common reaction
%path (CRP) method.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_CRP(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               MCInitPert = the starting number of compartments for the perturbed process.  Integer.
%               Xinit = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               XinitPert = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for perturbed process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               parametersPerturbed = the parameters of the perturbed model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     MCPert = number giving the ending size of the compartment count for perturbed process
%
%                     XPert = state of the perturbed system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%We simulate the compartment events a single time so the processes are coupled completely in that way.  
%We then simulate the various compartments between those events.
%
%Written by David F. Anderson, Nobember 24th, 2024.


function [MC, X, XPertk1, XPertk2, XPertk3, XPertdM, XPertdP, XPertdD, NRV] = SingleRealizationSingleTimeCompShell_CRP(MCInit, Xinit, XinitPertk1, XinitPertk2, XinitPertk3, XinitPertdM, XinitPertdP, XinitPertdD, parameters, PertFactor,tend)
%THIS ONLY WORKS IF THE PERTURBED PARAMETER IS ONE OF THE CHEMISTRY
%ONES!!!!!!!

%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^4;


%State of the system.  I just need X and all the perturbed processes.
X = Xinit;

XPertk1 = XinitPertk1;
XPertk2 = XinitPertk2;
XPertk3 = XinitPertk3;
XPertdM = XinitPertdM;
XPertdP = XinitPertdP;
XPertdD = XinitPertdD;

MC = MCInit;

%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
% k1 = parameters(5);
% k2 = parameters(6);
% k3 = parameters(7);
% dM = parameters(8);
% dP = parameters(9);
% dD = parameters(10);


%Rates for the compartments.
lambdaComp = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input
    lambdaComp(1) = KI;
    %Rate of compartment exit
    lambdaComp(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C. 
    lambdaComp(3) = KF*MC;
    %Rate of Coagulation
    lambdaComp(4) = KC*MC*(MC-1);

    %Total rate for compartments
    TotalRate = lambdaComp(1)+lambdaComp(2)+lambdaComp(3)+lambdaComp(4);
    
    %Get time of next compartment event.  Standard method.
    r1 = rand;
    NRV=NRV+1;

    Delta = log(1/r1)/TotalRate;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.
    %If so, we still need to simulate up to the end time.
    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %At this point, we have MC compartments.  So need to call a CRP
    %algorithm for each of the different processes.  There are 7: the 
    % nominal process and the 6 perturbed processes. 

    for k = 1:MC
        %I need to send the RVs.  This should be plenty.  These are the
        %random variables I will use for EACH of the processes.  
        RVs = rand([6,50]);
        RVs = log(1./RVs);


        %This gets me the nominal process and I only need to do it this one
        %time.
        [x,nrv1] = NRM_CRP(times(i+1)-times(i),X(:,k),parameters,RVs);
        X(:,k) = x;
        NRV = NRV + nrv1;

        %k1
        parametersPerturbed = parameters;
        parametersPerturbed(5) = (1+PertFactor)*parameters(5);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertk1(:,k),parametersPerturbed,RVs);
        XPertk1(:,k) = xPert;
        NRV=NRV+ nrv2;

        %k2
        parametersPerturbed = parameters;
        parametersPerturbed(6) = (1+PertFactor)*parameters(6);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertk2(:,k),parametersPerturbed,RVs);
        XPertk2(:,k) = xPert;
        NRV=NRV+ nrv2;

        %k3
        parametersPerturbed = parameters;
        parametersPerturbed(7) = (1+PertFactor)*parameters(7);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertk3(:,k),parametersPerturbed,RVs);
        XPertk3(:,k) = xPert;
        NRV=NRV+ nrv2;

        %dM
        parametersPerturbed = parameters;
        parametersPerturbed(8) = (1+PertFactor)*parameters(8);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertdM(:,k),parametersPerturbed,RVs);
        XPertdM(:,k) = xPert;
        NRV=NRV+ nrv2;

        %dP
        parametersPerturbed = parameters;
        parametersPerturbed(9) = (1+PertFactor)*parameters(9);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertdP(:,k),parametersPerturbed,RVs);
        XPertdP(:,k) = xPert;
        NRV=NRV+ nrv2;

        %dD
        parametersPerturbed = parameters;
        parametersPerturbed(10) = (1+PertFactor)*parameters(10);

        [xPert,nrv2] = NRM_CRP(times(i+1)-times(i),XPertdD(:,k),parametersPerturbed,RVs);
        XPertdD(:,k) = xPert;
        NRV=NRV+ nrv2;

    end


    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %We are done if this happens.
        break
    else
        
        %Not done.  See what happened with the compartments and update
        r2=rand;
        NRV = NRV+1;

        if r2 < lambdaComp(1)/TotalRate %appearance of compartment at end

            
            [X] = Entry(X,MC);
            [XPertk1] = Entry(XPertk1,MC);
            [XPertk2] = Entry(XPertk2,MC);
            [XPertk3] = Entry(XPertk3,MC);
            [XPertdM] = Entry(XPertdM,MC);
            [XPertdP] = Entry(XPertdP,MC);
            [XPertdD] = Entry(XPertdD,MC);
            
            %Update number of compartments
            MC = MC+1;

            

        elseif r2 < (lambdaComp(1)+lambdaComp(2))/TotalRate %deletion

            
            %find the one to be deleted.
            IndexDel = randi([1,MC]);
            NRV=NRV+1;
            
            [X] = Del(X,MC,IndexDel);
            [XPertk1] = Del(XPertk1,MC,IndexDel);
            [XPertk2] = Del(XPertk2,MC,IndexDel);
            [XPertk3] = Del(XPertk3,MC,IndexDel);
            [XPertdM] = Del(XPertdM,MC,IndexDel);
            [XPertdP] = Del(XPertdP,MC,IndexDel);
            [XPertdD] = Del(XPertdD,MC,IndexDel);

            %Update number of compartments
            MC = MC-1;


        elseif r2<(lambdaComp(1)+lambdaComp(2)+lambdaComp(3))/TotalRate %fragmentation.


            
            %Pick which compartment fragments.  Note state dependent, so easy.
            IndexFrag = randi([1,MC]);
            NRV=NRV+1;

            %%%%HYPERGEOMETRIC OPTION HERE
            %I think I need to handle X first, and then call the others.
            MIndexNominal = binornd(X(1,IndexFrag),1/2);
            PIndexNominal = binornd(X(2,IndexFrag),1/2);
            DIndexNominal = binornd(X(3,IndexFrag),1/2);
            NRV = NRV+3;
            
            NRV = NRV + 6*2;
            %Now to get the others.
            [XPertk1] = fragHyper(XPertk1,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);
            [XPertk2] = fragHyper(XPertk2,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);
            [XPertk3] = fragHyper(XPertk3,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);
            [XPertdM] = fragHyper(XPertdM,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);
            [XPertdP] = fragHyper(XPertdP,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);
            [XPertdD] = fragHyper(XPertdD,X,MC,IndexFrag,MIndexNominal,PIndexNominal,DIndexNominal);

            %Add new compartment to nominal process and update new compartment
            X(1,MC+1) = X(1,IndexFrag)-MIndexNominal;
            X(2,MC+1) = X(2,IndexFrag)-PIndexNominal;
            X(3,MC+1) = X(3,IndexFrag)-DIndexNominal;

            %update IdexFrag
            X(1,IndexFrag) = MIndexNominal;
            X(2,IndexFrag) = PIndexNominal;
            X(3,IndexFrag) = DIndexNominal;

            %%%NO COUPLING OPTION HERE
            % [X] = frag(X,MC,IndexFrag);
            % [XPertk1] = frag(XPertk1,MC,IndexFrag);
            % [XPertk2] = frag(XPertk2,MC,IndexFrag);
            % [XPertk3] = frag(XPertk3,MC,IndexFrag);
            % [XPertdM] = frag(XPertdM,MC,IndexFrag);
            % [XPertdP] = frag(XPertdP,MC,IndexFrag);
            % [XPertdD] = frag(XPertdD,MC,IndexFrag);


            MC = MC+1;

        else %Do coagulation

            
            %select two indices at random
            ri1=randi([1 MC]);
            ri2=randi([1 MC]);
            NRV=NRV+2;

            while ri1==ri2
                ri2=randi([1 MC]);
                NRV=NRV+1;
            end
            %I will put the material in ri1;

            [X] = coag(X,MC,ri1,ri2);
            [XPertk1] = coag(XPertk1,MC,ri1,ri2);
            [XPertk2] = coag(XPertk2,MC,ri1,ri2);
            [XPertk3] = coag(XPertk3,MC,ri1,ri2);
            [XPertdM] = coag(XPertdM,MC,ri1,ri2);
            [XPertdP] = coag(XPertdP,MC,ri1,ri2);
            [XPertdD] = coag(XPertdD,MC,ri1,ri2);
            %update number of compartments.
            MC = MC-1;


        end

    end
end



%
% MoutputPlot=zeros([number_timepoints+1,1]);
% PoutputPlot=zeros([number_timepoints+1,1]);
% DoutputPlot=zeros([number_timepoints+1,1]);
% for ell = 1:number_timepoints+1
%     MoutputPlot(ell) = Xoutput(1,1,ell);
%     PoutputPlot(ell) = Xoutput(2,1,ell);
%     DoutputPlot(ell) = Xoutput(3,1,ell);
% end
% figure
% plot(OutputTimes,MoutputPlot,'blue',OutputTimes, PoutputPlot,'green', OutputTimes,DoutputPlot,'red')
% legend('M','P','D');


    function [Xnew] = Entry(X,M)
        %All previous compartments are the same
        Xnew = X;

        %New compartment with nothing in it.  Might need to change this
        %sometime.
        Xnew(:,M+1) = zeros([3,1]);
    end

    function [Xnew] = Del(X,M,ID)
        Xnew=X;

        %Now shift everything down
        for ell = ID:M-1
            Xnew(:,ell) = X(:,ell+1);
        end
    end

    function [XPNew] = fragHyper(XP,X,M,IF,MInd,PInd,DInd)


        XPNew = XP;

        if X(1,IF) <= XP(1,IF)
            MIndexPert = MInd + binornd(XP(1,IF)-X(1,IF),1/2);
        else
            MIndexPert = hygernd(X(1,IF),MInd,XP(1,IF));
        end

        if X(2,IF) <= XP(2,IF)
            PIndexPert = PInd + binornd(XP(2,IF)-X(2,IF),1/2);
        else
            PIndexPert = hygernd(X(2,IF),PInd,XP(2,IF));
        end

        if X(3,IF) <= XP(3,IF)
            DIndexPert = DInd + binornd(XP(3,IF)-X(3,IF),1/2);
        else
            DIndexPert = hygernd(X(3,IF),DInd,XP(3,IF));
        end


        XPNew(1,M+1) = XP(1,IF)-MIndexPert;
        XPNew(2,M+1) = XP(2,IF)-PIndexPert;
        XPNew(3,M+1) = XP(3,IF)-DIndexPert;

        XPNew(1,IF) = MIndexPert;
        XPNew(2,IF) = PIndexPert;
        XPNew(3,IF) = DIndexPert;


    end

    function [Xnew] = frag(X,M,IF)

        Xnew = X;

        %I now fragment. These are the molecules staying.  The step below
        %is commented out because it is the worst coupling.
        MIndex = binornd(X(1,IF),1/2);
        PIndex = binornd(X(2,IF),1/2);
        DIndex = binornd(X(3,IF),1/2);


        %Add new compartment and update new compartment
        Xnew(1,M+1) = X(1,IF)-MIndex;
        Xnew(2,M+1) = X(2,IF)-PIndex;
        Xnew(3,M+1) = X(3,IF)-DIndex;

        %update IdexFrag
        Xnew(1,IF) = MIndex;
        Xnew(2,IF) = PIndex;
        Xnew(3,IF) = DIndex;

    end


    function [Xnew] = coag(X,M,r1,r2)

        Xnew = X;

        %collect in r1 compartment.
        Xnew(:,r1)=Xnew(:,r1)+Xnew(:,r2);

        %Now I have to delete ri2 and shift everything down.
        for ell = r2:M-1
            Xnew(:,ell) = Xnew(:,ell+1);
        end
    end



end
