%The code produces a single realization of coupled paths for the following RNIC model.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MCoutput,Xoutput,XoutputPert,NRV]= SingleRealizationFullPath_Shell_Split(parametersPerturbed, parameters,tend,CompartmentMax,number_timepoints)
%
%
%               parametersPerturbed = the perturbed parameters of the model (listed above). Vector.
%               parameters = the nominal parameters of the model (listed above). Vector.
%               tend = end time. Real.
%               CompartmentMax = maximum number of compartments.  Integer.
%               number_timepoints = total number of timepoints for the purposes of plotting.  Integer.
%
%
%The outpus are the following:
%
%                     MCoutput = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time points.
%
%                     Xoutput = state of system at relevant times for nominal model.  Vector of size
%                     3 x CompartmentMax x (number_timepoints + 1)
%
%                     XoutputPert = state of system at relevant times for perturbed model.  Vector of size
%                     3 x CompartmentMax x (number_timepoints + 1)
%
%                     NRV = number of random variables utilized.
%
%Written by David F. Anderson, November 24th, 2024.


function [MCoutput, Xnomk1, Xpertk1, Xnomk2, Xpertk2,Xnomk3,Xpertk3, XnomdM,XpertdM, XnomdP,XpertdP,XnomdD, XpertdD, NRV]= SingleRealizationFullPath_Shell_Split(PertFactor, parameters,tend,CompartmentMax,number_timepoints)

%Need to have the times that I will be updating my outputs. This is the
%amount of time I must simulate.
del = tend/number_timepoints;
OutputTimes = zeros([number_timepoints+1,1]);
for ell = 1:number_timepoints
    OutputTimes(ell+1) = OutputTimes(ell) + del;
end

%State of the system at relevant time-points.
Xnomk1 = zeros([3,CompartmentMax,number_timepoints+1]);
Xpertk1 = zeros([3,CompartmentMax,number_timepoints+1]);
Xnomk2 = zeros([3, CompartmentMax, number_timepoints+1]);
Xpertk2 = zeros([3, CompartmentMax, number_timepoints+1]);
Xnomk3 = zeros([3, CompartmentMax, number_timepoints+1]);
Xpertk3 = zeros([3, CompartmentMax, number_timepoints+1]);
XnomdM = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdM = zeros([3, CompartmentMax, number_timepoints+1]);
XnomdP = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdP = zeros([3, CompartmentMax, number_timepoints+1]);
XnomdD = zeros([3, CompartmentMax, number_timepoints+1]);
XpertdD = zeros([3, CompartmentMax, number_timepoints+1]);

MCoutput = zeros([number_timepoints+1,1]);

%INITIALIZE
MCoutput(1)=1;

Xinitial=zeros([3,1]);
XPertInitial=zeros([3,1]);

%initialize all the processes
for k = 1:MCoutput(1)
    Xnomk1(:,k,1) = Xinitial;
    Xnomk2(:,k,1) = Xinitial;
    Xnomk3(:,k,1) = Xinitial;
    XnomdM(:,k,1) = Xinitial;
    XnomdP(:,k,1) = Xinitial;
    XnomdD(:,k,1) = Xinitial;
end
for k = 1:MCoutput(1)
    Xpertk1(:,k,1) = XPertInitial;
    Xpertk2(:,k,1) = XPertInitial;
    Xpertk3(:,k,1) = XPertInitial;
    XpertdM(:,k,1) = XPertInitial;
    XpertdP(:,k,1) = XPertInitial;
    XpertdD(:,k,1) = XPertInitial;
end

%How many random variables used
NRV=0;

%Main Loop to get the coupled paths.
for ell = 1:number_timepoints
    %MCoutput_new should be a single number
    %Xoutput_newk1, etc., should be a vector of size 3 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %XoutputPert_newk1, etc., should be a vector of size 3 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %NRV_new is the number of random variables used in this single step (of
    %size del)
    [MCoutput_new, Xoutput_newk1, XoutputPert_newk1, Xoutput_newk2, XoutputPert_newk2, Xoutput_newk3, XoutputPert_newk3, Xoutput_newdM, XoutputPert_newdM, Xoutput_newdP, XoutputPert_newdP, Xoutput_newdD, XoutputPert_newdD, NRV_new] = SingleRealizationSingleTimeCompShell_Split(MCoutput(ell), Xnomk1(:,:,ell), Xpertk1(:,:,ell), Xnomk2(:,:,ell), Xpertk2(:,:,ell), Xnomk3(:,:,ell), Xpertk3(:,:,ell), XnomdM(:,:,ell), XpertdM(:,:,ell), XnomdP(:,:,ell), XpertdP(:,:,ell), XnomdD(:,:,ell), XpertdD(:,:,ell), parameters, PertFactor, del);
    
    MCoutput(ell+1)=MCoutput_new;
    
    for k = 1:MCoutput_new
        Xnomk1(:,k,ell+1) = Xoutput_newk1(:,k);
        Xnomk2(:,k,ell+1) = Xoutput_newk2(:,k);
        Xnomk3(:,k,ell+1) = Xoutput_newk3(:,k);
        XnomdM(:,k,ell+1) = Xoutput_newdM(:,k);
        XnomdP(:,k,ell+1) = Xoutput_newdP(:,k);
        XnomdD(:,k,ell+1) = Xoutput_newdD(:,k);
    end
    for k = 1:MCoutput_new
        Xpertk1(:,k,ell+1) = XoutputPert_newk1(:,k);
        Xpertk2(:,k,ell+1) = XoutputPert_newk2(:,k);
        Xpertk3(:,k,ell+1) = XoutputPert_newk3(:,k);
        XpertdM(:,k,ell+1) = XoutputPert_newdM(:,k);
        XpertdP(:,k,ell+1) = XoutputPert_newdP(:,k);
        XpertdD(:,k,ell+1) = XoutputPert_newdD(:,k);
    end
    NRV = NRV + NRV_new;
end


