%The code produces a single realization of coupled paths for the following RNIC model.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MC,X,MCPert,XPert,NRV]= SingleRealizationSingleTimeCompShell_Split(MCInit,Xinit,MCInitPert,XinitPert,parameters,parametersPerturbed,tend)
%
%               MCInit = the starting number of compartments for the nominal process.  Integer.
%               MCInitPert = the starting number of compartments for the perturbed process.  Integer.
%               Xinit = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for nominal process
%               XinitPert = vector of size 3 x (CompartmentMax) giving initial
%                       condition (with MCInit) for perturbed process
%               parameters = the parameters of the nominal model (listed above). Vector.
%               parametersPerturbed = the parameters of the perturbed model (listed above). Vector.
%               tend = end time. It is tend/number_timepoints from previous, or del. Real.
%
%
%The outpus are the following:
%
%                     MC = number giving the ending size of the compartment count for nominal process
%
%                     X = state of the nominal system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     MCPert = number giving the ending size of the compartment count for perturbed process
%
%                     XPert = state of the perturbed system at end time.  Vector of size
%                     3 x CompartmentMax 
%
%                     NRV = number of random variables utilized.
%
%We simulate the compartment events a single time so the processes are coupled completely in that way.  
%We then simulate the various compartments between those events.
%
%Written by David F. Anderson, November  24th, 2024.


function [MC, Xk1, XPertk1, Xk2, XPertk2, Xk3, XPertk3, XdM, XPertdM, XdP, XPertdP, XdD, XPertdD, NRV] = SingleRealizationSingleTimeCompShell_Split(MCInit, Xinitk1, XinitPertk1, Xinitk2, XinitPertk2, Xinitk3, XinitPertk3, XinitdM, XinitPertdM, XinitdP, XinitPertdP, XinitdD, XinitPertdD, parameters, PertFactor,tend)
%THIS ONLY WORKS IF THE PERTURBED PARAMETER IS ONE OF THE CHEMISTRY
%ONES!!!!!!!

%NMax will be the maximum possible number of steps.  We shouldn't touch it.
NMax = 10^4;


%State of the system.
Xk1 = Xinitk1;
Xk2 = Xinitk2;
Xk3 = Xinitk3;
XdM = XinitdM;
XdP = XinitdP;
XdD = XinitdD;

XPertk1 = XinitPertk1;
XPertk2 = XinitPertk2;
XPertk3 = XinitPertk3;
XPertdM = XinitPertdM;
XPertdP = XinitPertdP;
XPertdD = XinitPertdD;

MC = MCInit;



%Set parameters for the compartment model.  Don't need chemistry yet.
KI = parameters(1);
KE = parameters(2);
KF = parameters(3);
KC = parameters(4);
% k1 = parameters(5);
% k2 = parameters(6);
% k3 = parameters(7);
% dM = parameters(8);
% dP = parameters(9);
% dD = parameters(10);


%Rates for the compartments.
lambdaComp = zeros([4,1]);

%Need to keep track of times.
times=zeros([1,NMax+1]);

NRV=0;

%Use this to know when to stop.
CheckBreak=0;
for i = 1:NMax
    
    %Rate of compartment input
    lambdaComp(1) = KI;
    %Rate of compartment exit
    lambdaComp(2) = KE*MC;
    %Rate of fragmentation, C -[KF]-> 2C. 
    lambdaComp(3) = KF*MC;
    %Rate of Coagulation
    lambdaComp(4) = KC*MC*(MC-1);

    %Total rate for compartments
    TotalRate = lambdaComp(1)+lambdaComp(2)+lambdaComp(3)+lambdaComp(4);
    
    %Get time of next compartment event.  Standard method.

    r1 = rand;
    NRV=NRV+1;

    % if i == 1
    %     r1
    % end

    Delta = log(1/r1)/TotalRate;
    times(i+1)=times(i)+Delta;
    
    %check if we went passed end time.
    %If so, we still need to simulate up to the end time.
    if times(i+1) > tend
        times(i+1)=tend;
        CheckBreak=1;
    end
    
    
    %At this point, we have MC compartments.  So need to call a split
    %coupling function to get the two paths at the new end time for each of 
    % the different coupled processes (i.e., each parameter choice).  This can
    %be modified to use CRN, Independent paths, or the common reaction path
    %method quite easily.  THIS IS THE ONLY THING THAT NEEDS TO CHANGE FOR
    %ANY OF THESE METHODS.

    %k1
    parametersPerturbed = parameters;
    parametersPerturbed(5) = (1+PertFactor)*parameters(5);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),Xk1(:,k),XPertk1(:,k),parameters,parametersPerturbed);
        Xk1(:,k) = x;
        XPertk1(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %k2
    parametersPerturbed = parameters;
    parametersPerturbed(6) = (1+PertFactor)*parameters(6);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),Xk2(:,k),XPertk2(:,k),parameters,parametersPerturbed);
        Xk2(:,k) = x;
        XPertk2(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %k3
    parametersPerturbed = parameters;
    parametersPerturbed(7) = (1+PertFactor)*parameters(7);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),Xk3(:,k),XPertk3(:,k),parameters,parametersPerturbed);
        Xk3(:,k) = x;
        XPertk3(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %dM
    parametersPerturbed = parameters;
    parametersPerturbed(8) = (1+PertFactor)*parameters(8);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),XdM(:,k),XPertdM(:,k),parameters,parametersPerturbed);
        XdM(:,k) = x;
        XPertdM(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %dP
    parametersPerturbed = parameters;
    parametersPerturbed(9) = (1+PertFactor)*parameters(9);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),XdP(:,k),XPertdP(:,k),parameters,parametersPerturbed);
        XdP(:,k) = x;
        XPertdP(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %dD
    parametersPerturbed = parameters;
    parametersPerturbed(10) = (1+PertFactor)*parameters(10);
    for k = 1:MC
        [x,xPert,nrvs] = StandardGillespieSplit(times(i+1)-times(i),XdD(:,k),XPertdD(:,k),parameters,parametersPerturbed);
        XdD(:,k) = x;
        XPertdD(:,k) = xPert;
        NRV=NRV+nrvs;
    end

    %Now to determine the compartment update at the next time if we are not
    %simply finished.

    if CheckBreak == 1
        %We are done if this happens.
        break
    else
        
        %Not done.  See what happened with the compartments and update
        r2=rand;
        NRV = NRV+1;

        if r2 < lambdaComp(1)/TotalRate %appearance of compartment at end

            [Xk1,XPertk1] = Entry(Xk1,XPertk1,MC);
            [Xk2,XPertk2] = Entry(Xk2,XPertk2,MC);
            [Xk3,XPertk3] = Entry(Xk3,XPertk3,MC);
            [XdM,XPertdM] = Entry(XdM,XPertdM,MC);
            [XdP,XPertdP] = Entry(XdP,XPertdP,MC);
            [XdD,XPertdD] = Entry(XdD,XPertdD,MC);
            
            %Update number of compartments
            MC = MC+1;

        elseif r2 < (lambdaComp(1)+lambdaComp(2))/TotalRate %deletion
            %find the one to be deleted.
            IndexDel = randi([1,MC]);
            NRV=NRV+1;
            
            [Xk1,XPertk1] = Del(Xk1,XPertk1,MC,IndexDel);
            [Xk2,XPertk2] = Del(Xk2,XPertk2,MC,IndexDel);
            [Xk3,XPertk3] = Del(Xk3,XPertk3,MC,IndexDel);
            [XdM,XPertdM] = Del(XdM,XPertdM,MC,IndexDel);
            [XdP,XPertdP] = Del(XdP,XPertdP,MC,IndexDel);
            [XdD,XPertdD] = Del(XdD,XPertdD,MC,IndexDel);

            %Update number of compartments
            MC = MC-1;


        elseif r2<(lambdaComp(1)+lambdaComp(2)+lambdaComp(3))/TotalRate %fragmentation.


            %Pick which compartment fragments.  Note state dependent, so easy.
            IndexFrag = randi([1,MC]);
            NRV=NRV+1;

            %Each call to frag uses 2*3=6 RVs.  There are then 6 calls.
            NRV = NRV+6*6;

            [Xk1,XPertk1] = frag(Xk1,XPertk1,MC,IndexFrag);
            [Xk2,XPertk2] = frag(Xk2,XPertk2,MC,IndexFrag);
            [Xk3,XPertk3] = frag(Xk3,XPertk3,MC,IndexFrag);
            [XdM,XPertdM] = frag(XdM,XPertdM,MC,IndexFrag);
            [XdP,XPertdP] = frag(XdP,XPertdP,MC,IndexFrag);
            [XdD,XPertdD] = frag(XdD,XPertdD,MC,IndexFrag);
            

            MC = MC+1;

        else %Do coagulation

            %select two indices at random
            ri1=randi([1 MC]);
            ri2=randi([1 MC]);
            NRV=NRV+2;

            while ri1==ri2
                ri2=randi([1 MC]);
                NRV=NRV+1;
            end
            %I will put the material in ri1;

            [Xk1,XPertk1] = coag(Xk1,XPertk1,MC,ri1,ri2);
            [Xk2,XPertk2] = coag(Xk2,XPertk2,MC,ri1,ri2);
            [Xk3,XPertk3] = coag(Xk3,XPertk3,MC,ri1,ri2);
            [XdM,XPertdM] = coag(XdM,XPertdM,MC,ri1,ri2);
            [XdP,XPertdP] = coag(XdP,XPertdP,MC,ri1,ri2);
            [XdD,XPertdD] = coag(XdD,XPertdD,MC,ri1,ri2);

            %update number of compartments.
            MC = MC-1;

        end

    end
end


% 
% MoutputPlot=zeros([number_timepoints+1,1]);
% PoutputPlot=zeros([number_timepoints+1,1]);
% DoutputPlot=zeros([number_timepoints+1,1]);
% for ell = 1:number_timepoints+1
%     MoutputPlot(ell) = Xoutput(1,1,ell);
%     PoutputPlot(ell) = Xoutput(2,1,ell);
%     DoutputPlot(ell) = Xoutput(3,1,ell);
% end
% figure
% plot(OutputTimes,MoutputPlot,'blue',OutputTimes, PoutputPlot,'green', OutputTimes,DoutputPlot,'red')
% legend('M','P','D');

    function [Xnew,XPertnew] = Entry(X,XPert,M)
        %All previous compartments are the same
        Xnew = X;
        XPertnew = XPert;
        %New compartment with nothing in it.  Might need to change this
        %sometime.
        Xnew(:,M+1) = zeros([3,1]);
        XPertnew(:,M+1) = zeros([3,1]);
    end

    function [Xnew,XPertnew] = Del(X,XPert,M,ID)
        Xnew=X;
        XPertnew=XPert;
        % %update all before this index
        % for ell = 1:ID-1
        %     Xnew(:,ell) = X(:,ell);
        %     XPertnew(:,ell) = XPert(:,ell);
        % end
        %Now shift everything down
        for ell = ID:M-1
            Xnew(:,ell) = X(:,ell+1);
            XPertnew(:,ell) = XPert(:,ell+1);
        end
    end

    function [Xnew,XPertnew] = frag(X,XPert,M,IF)

        Xnew = X;
        XPertnew = XPert;
        %I want to couple these as tightly as possible.
        if X(1,IF) > XPert(1,IF)
            MIndexPert = binornd(XPert(1,IF),1/2);
            MIndex = MIndexPert + binornd(X(1,IF)-XPert(1,IF),1/2);
        else
            MIndex = binornd(X(1,IF),1/2);
            MIndexPert = MIndex + binornd(XPert(1,IF)-X(1,IF),1/2);
        end

        if X(2,IF) > XPert(2,IF)
            PIndexPert = binornd(XPert(2,IF),1/2);
            PIndex = PIndexPert + binornd(X(2,IF)-XPert(2,IF),1/2);
        else
            PIndex = binornd(X(2,IF),1/2);
            PIndexPert = PIndex + binornd(XPert(2,IF)-X(2,IF),1/2);
        end

        if X(3,IF) > XPert(3,IF)
            DIndexPert = binornd(XPert(3,IF),1/2);
            DIndex = DIndexPert + binornd(X(3,IF)-XPert(3,IF),1/2);
        else
            DIndex = binornd(X(3,IF),1/2);
            DIndexPert = DIndex + binornd(XPert(3,IF)-X(3,IF),1/2);
        end

        % %update all compartments to be exactly the same
        %NOT NEEDED
        % for ell = 1:M
        %     Xnew(:,ell) = X(:,ell);
        %     XPertnew(:,ell) = XPert(:,ell);
        % end

        %Add new compartment and update new compartment
        Xnew(1,M+1) = X(1,IF)-MIndex;
        Xnew(2,M+1) = X(2,IF)-PIndex;
        Xnew(3,M+1) = X(3,IF)-DIndex;

        XPertnew(1,M+1) = XPert(1,IF)-MIndexPert;
        XPertnew(2,M+1) = XPert(2,IF)-PIndexPert;
        XPertnew(3,M+1) = XPert(3,IF)-DIndexPert;

        %update IdexFrag
        Xnew(1,IF) = MIndex;
        Xnew(2,IF) = PIndex;
        Xnew(3,IF) = DIndex;

        XPertnew(1,IF) = MIndexPert;
        XPertnew(2,IF) = PIndexPert;
        XPertnew(3,IF) = DIndexPert;
    end

    function [Xnew,XPertnew] = coag(X,XPert,M,r1,r2)

        Xnew = X;
        XPertnew = XPert;
        %First, update all to be the same
        % for ell = 1:M
        %     Xnew(:,ell) = X(:,ell);
        %     XPertnew(:,ell) = XPert(:,ell);
        % end

        %collect in r1 compartment.
        Xnew(:,r1)=Xnew(:,r1)+Xnew(:,r2);
        XPertnew(:,r1) = XPertnew(:,r1)+XPertnew(:,r2);

        %Now I have to delete ri2 and shift everything down.
        for ell = r2:M-1
            Xnew(:,ell) = Xnew(:,ell+1);
            XPertnew(:,ell) = XPertnew(:,ell+1);
        end
    end



end