%The code produces coupled realizations of the following chemical model.
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% with both nominal and perturbed parameters.  
%
%Usage [x,xPert,nrv]= StandardGillespieSplit(tend,xin,xPertin, parameters,parametersPerturbed)
%
%
%               tend = end time. Real.
%               xin = initial condition for nominal process.  3 x 1 vector.
%               xPertin = initial condition for perturbed process.  3 x 1 vector.
%               
%               parameters = the parameters of the model.
%               Vector.  Only use 5 - 10
%
%               parametersPertubed = the parameters of the perturbed model.
%               Vector.  Only use 5 - 10
%
%
%The outpus are the following:
%
%                     x = output vector (final state) for nominal process.
%                     xPert = output vector (final state) for perturbed process.
%
%                     nrv = number of random variables utilized.
%
%Written by David F. Anderson on June 4th, 2024.

function [x,xPert,nrv]= StandardGillespieSplit(tend,xin,xPertin, parameters,parametersPerturbed)

%SPLIT COUPLING.

%max number of steps
N=10^4;

k1 = parameters(5);
k2 = parameters(6);
k3 = parameters(7);
dM = parameters(8);
dP = parameters(9);
dD = parameters(10);

k1p = parametersPerturbed(5);
k2p = parametersPerturbed(6);
k3p = parametersPerturbed(7);
dMp = parametersPerturbed(8);
dPp = parametersPerturbed(9);
dDp = parametersPerturbed(10);

%Rates for the (split) coupled processes (3 for each process)
lambda = zeros([6,3]);

%Placeholders for sums of lambda(i,:) (so can use Gillespie implementation)
A = zeros([6,1]);

%Rates for nominal process
a = zeros([6,1]);

%Rates for perturbed process
b = zeros([6,1]);

%Initialize
x = xin;
xPert = xPertin;

times=zeros([1,N+1]);
nrv=0;

%main loop
for i = 1:N
    
    %intensities for nominal process
    a(1) = k1;
    a(2) = dM*x(1);
    a(3) = k2*x(1);
    a(4) = dP*x(2);
    a(5) = k3*x(2)*(x(2)-1);
    a(6) = dD*x(3);
    
    %intensities for perturbed process.
    b(1) = k1p;
    b(2) = dMp*xPert(1);
    b(3) = k2p*xPert(1);
    b(4) = dPp*xPert(2);
    b(5) = k3p*xPert(2)*(xPert(2)-1);
    b(6) = dDp*xPert(3);
    
    %intensities for coupled process.  First is min, then extra for
    %nominal, then extra for perturbed.  Finally, A(k) gives sum (max)
    for k = 1:6
        lambda(k,1) = min(a(k),b(k));
        lambda(k,2) = a(k) - lambda(k,1);
        lambda(k,3) = b(k) - lambda(k,1);
        A(k) = lambda(k,1)+lambda(k,2)+lambda(k,3);
    end
    
    %Finding total intensities. 
    lambda0 = 0;
    for k = 1:6
        lambda0 = lambda0 + A(k);
    end
    
    r = rand;
    nrv=nrv+1;
    
    %Time until next reaction
    Delta = log(1/r)/lambda0;
    
    times(1,i+1)=times(1,i)+Delta;
    if times(1,i+1)>tend
        break
    else
        
        r2=rand;
        nrv=nrv+1;
             
        %Find the overall reaction that happened.  Then discover which one.
        
        if r2<A(1)/lambda0 %reaction 1, 0 --> M
            if r2 < lambda(1,1)/lambda0 %update both
                x(1) = x(1) + 1;
                xPert(1) = xPert(1) + 1;
            elseif r2 < (lambda(1,1) + lambda(1,2))/lambda0 %update nominal
                x(1) = x(1) + 1;
            else %update perturbed
                xPert(1) = xPert(1) + 1;
            end
                        
        elseif r2<(A(1) + A(2))/lambda0 %reaction 2, M --> 0
            if r2 < (A(1) + lambda(2,1))/lambda0 %update both
                x(1) = x(1) - 1;
                xPert(1) = xPert(1) - 1;
            elseif r2 < (A(1) + lambda(2,1) + lambda(2,2))/lambda0 %update nominal
                x(1) = x(1) - 1;
            else %update perturbed
                xPert(1) = xPert(1) - 1;
            end
            
        elseif r2<(A(1) + A(2) + A(3))/lambda0 %reaction 3, M --> M + P
            if r2<(A(1) + A(2) + lambda(3,1))/lambda0 %update both
                x(2) = x(2) + 1;
                xPert(2) = xPert(2) + 1;
            elseif r2<(A(1) + A(2) + lambda(3,1) + lambda(3,2))/lambda0 %update nominal
                x(2) = x(2) + 1;
            else %update perturbed
                xPert(2) = xPert(2) + 1;
            end
            
        elseif r2 < (A(1) + A(2) + A(3) + A(4))/lambda0 %reaction 4, P --> 0
            if r2 < (A(1) + A(2) + A(3) + lambda(4,1))/lambda0 %update both 
                x(2) = x(2) - 1;
                xPert(2) = xPert(2) - 1;
            elseif r2 < (A(1) + A(2) + A(3) + lambda(4,1) + lambda(4,2))/lambda0 %update nominal
                x(2) = x(2) - 1;
            else %update perturbed
                xPert(2) = xPert(2) - 1;
            end
            
        elseif r2<(A(1) + A(2) + A(3) + A(4) + A(5))/lambda0 %reaction 5, 2P --> D
            if r2<(A(1) + A(2) + A(3) + A(4) + lambda(5,1))/lambda0 %update both
                x(2) = x(2) - 2;
                xPert(2) = xPert(2) - 2;
                x(3) = x(3) + 1;
                xPert(3) = xPert(3) + 1;
            elseif r2<(A(1) + A(2) + A(3) + A(4) + lambda(5,1) + lambda(5,2))/lambda0 %update nominal
                x(2) = x(2) - 2;
                x(3) = x(3) + 1;
            else %update perturbed
                xPert(2) = xPert(2) - 2;
                xPert(3) = xPert(3) + 1;
            end
                
        else %reaction 6, D --> 0
            if r2<(A(1) + A(2) + A(3) + A(4) + A(5) + lambda(6,1))/lambda0 %update both
                x(3) = x(3) - 1;
                xPert(3) = xPert(3) - 1;
            elseif r2<(A(1) + A(2) + A(3) + A(4) + A(5) + lambda(6,1) + lambda(6,2))/lambda0 %update nominal
                x(3) = x(3) - 1;
            else %update perturbed
                xPert(3) = xPert(3) - 1;
            end
            
        end
    end
    
end

