%The code produces a single realization of a path for the following RNIC model.
%It utilizes a stream of unit-exponential random variables with the idea
%that these paths are coupled through the common reactino path methd.
%
%                   Compartments: 0 -[KI]-> C,  
%                                 C -[KE]-> 0, 
%                                 C -[KF]-> 2C,
%                                2C -[KC]-> C,
%
%
%                    Chemistry:  0 -[k1]-> M,
%                                M -[dM]-> 0,
%                                M -[k2]-> M + P
%                                P -[dP]-> 0 
%                               2P -[k3]-> D
%                                D -[dD]-> 0
%
%
% Usage [MCoutput,Xoutput,NRV]= SingleRealizationFullPath_Shell_CRP(parameters,tend,CompartmentMax,number_timepoints,RVs)
%
%
%               parameters = the nominal parameters of the model (listed above). Vector.
%               tend = end time. Real.
%               CompartmentMax = maximum number of compartments.  Integer.
%               number_timepoints = total number of timepoints for the purposes of plotting.  Integer.
%               RVsExp = (4 x many) array of unit exponentials to generate the
%               path.
%               RVsDelete = random variables used for deletion events.
%               RVsFrag = random variables used for fragmentation.
%               RVsCoag = random variables used for coagulation.
%
%The outpus are the following:
%
%                     MCoutput = vector of size (number_timepoints + 1) x 1
%                     giving the value of MC at different relevant time points.
%
%                     Xoutput = state of system at relevant times for nominal model.  Vector of size
%                     3 x CompartmentMax x (number_timepoints + 1)
%
%                     NRV = number of random variables utilized.
%
%Written by David F. Anderson, June 10th, 2024.

function [MCoutput,Xoutput,NRV]= SingleRealizationFullPath_Shell_CRP(parameters,tend,CompartmentMax,number_timepoints,RVsExp,RVsDelete,RVsFrag,RVsCoag,RVsBinom)

%Need to have the times that I will be updating my outputs. This is the
%amount of time I must simulate.
del = tend/number_timepoints;
OutputTimes = zeros([number_timepoints+1,1]);
for ell = 1:number_timepoints
    OutputTimes(ell+1) = OutputTimes(ell) + del;
end

%State of the system at relevant time-points.
Xoutput = zeros([3,CompartmentMax,number_timepoints+1]);

MCoutput = zeros([number_timepoints+1,1]);

%INITIALIZE
MCoutput(1)=1;

Xinitial=zeros([3,1]);

for k = 1:MCoutput(1)
    Xoutput(:,k,1) = Xinitial;
end

%How many random variables used
NRV=0;

%Pointer tells us what the next unit-exponential should be.
PointerExp = ones([4,1]);
%Pointer tells us what the next deletion RV should be.
PointerDel = 1;
%Pointer tells us what the next Fragmentation RV should be.
PointerFrag= 1;
%Pointer tells us what the next coagulation RV should be.
PointerCoag = 1;
PointerBinom = 1;

%Internal times of the integrated intensity functions.
T= zeros([4,1]);

%Times for the next unit exponential (Poisson)
P=zeros([4,1]);
for k = 1:4
    P(k) = RVsExp(k,PointerExp(k));
    PointerExp(k) = PointerExp(k)+1;
end


%Main Loop to get the coupled paths.
for ell = 1:number_timepoints
    %MCoutput_new should be a single number
    %Xoutput_new should be a vector of size 3 x CompartmentMax (with only
    %the first MCoutput_new columms mattering)
    %NRV_new is the number of random variables used in this single step (of
    %size del)
    [MCoutput_new,Xoutput_new,NRV_new,PointerExpNew,PointerDelNew,PointerFragNew,PointerCoagNew,PointerBinomNew,Tnew,Pnew]= SingleRealizationSingleTimeCompShell_CRP(MCoutput(ell),Xoutput(:,:,ell), parameters,del,RVsExp,RVsDelete,RVsFrag,RVsCoag, RVsBinom, PointerExp, PointerDel, PointerFrag, PointerCoag,PointerBinom, T,P);
    
    %update P and T
    P=Pnew;
    T=Tnew;
    %Update pointers
    PointerExp=PointerExpNew;
    PointerDel=PointerDelNew;
    PointerFrag=PointerFragNew;
    PointerCoag=PointerCoagNew;
    PointerBinom=PointerBinomNew;
    
    MCoutput(ell+1)=MCoutput_new;
    for k = 1:MCoutput_new
        Xoutput(:,k,ell+1) = Xoutput_new(:,k);
    end

    NRV = NRV + NRV_new;
end
% PointerExp
% PointerDel
% PointerFrag
% PointerCoag
% PointerBinom


